% This code computes the IRF  using a non-quadratic return function for a
% menu cost model as in Golosov and Lucas;  the code uses the steady state
% decision rules for computing the irf
clear all
close all

% parametrers
readparams;

%parameters for impulse response
N_sim      = 1e6/2; %number of firms simulated to obtain the invariant distribution
dt  =  1/52;   % 1/365/2   ;    % half a day, as in the  paper %0.01;
dM         = 0.01;            %percentage increase in money supply
time_shock = 1/10  ;          %time of the shock in the simulation of the IRF
T_end_irf  = time_shock+ 2;   %length of the response to the aggregate shock in years


% compute steady state rules
Solve_SS;

% Find the invariant distribution
Get_invariant;
save invariant_gaps_drift V_p
c_ss = (alpha*eta/(eta-1))^(-1/epsilon) * (phi_0'*exp((1-eta)*V_p))^(1/(epsilon*(eta-1)));

% Impulse responses
T_end      = T_end_irf; % last date of the history of each firm in years
Total_time = 0:dt:T_end; % in years
inde_time_shock = round(time_shock/dt) + 1; % in model periods

Tn         = length(Total_time);
sum_c      = zeros(Tn,1);       %  consumption  deviation from st st

for i=1:N_sim
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Object of interest
    p_tilde_t   = zeros(Tn,1); % log-price gap of a given firm at each point in time, after eventual replacement/adjustment    
 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % First period
    time      = 0; %actual time elapsed since beginning of simulations
    inde_time = 1; %indicator for model periods
    p_tilde   = V_p(i);  %  initial price gap  for firm i , computed by " Get_invariant " 
    
    %save the  object of interest
    p_tilde_t(inde_time)= p_tilde; % price gap at time t
    
    while inde_time<Tn
        time = time + dt;  % clock
        inde_time = inde_time+1; % model periods
        
        % check if before the observation date the firm dies, in which case do forced observation/adjustment
        omega=rand;
        
        %CASE A:Replacement
        if omega>=exp(-delta*dt)  %case of forced observation/adjustment
            p_tilde   = phat_ss;  %initial price gap
            
        else
            
            s = randn;
            p_tilde   =  p_tilde_t(inde_time-1)  + (gamma-mu)*dt + sigma*sqrt(dt)*s;
    %        Dp = 0 ;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % aggregate shock
            if inde_time == inde_time_shock,
                p_tilde = p_tilde-dM;
            end
            %Adjusting
            if (p_tilde<=low_p_ss ) || (p_tilde>=high_p_ss )
                
                p_tilde  = phat_ss;
                
            end
            
        end
        
        %save the 3 object of interest
        p_tilde_t(inde_time)= p_tilde; % update price gap

        
    end % loop over time for firm "i" 
    
    % compute aggregate c(t) by summing over the price gaps of firm c_(i,t)
    sum_c    = sum_c  + 1/N_sim * (alpha*eta/(eta-1) *exp((p_tilde_t))).^(1-eta);

end % loop over firms

c_prime =  sum_c.^(1/(epsilon*(eta-1)));
P_prime =  ( 1/alpha )*(c_prime ).^(-epsilon);



c_prime_pre=c_prime(1:inde_time_shock-1);
P_prime_pre=P_prime(1:inde_time_shock-1);


figure(1)
plot([1:1:length(c_prime_pre)]*dt,c_prime_pre)
title('steady state consumption')
xlabel('years')
grid('on')

figure(3)
plot(p_gg,phi_p)
title('invariant density of price gaps')
xlabel('price gaps')


c_prime_post=c_prime(inde_time_shock:inde_time_shock + 2/dt - 1);
P_prime_post=P_prime(inde_time_shock:inde_time_shock + 2/dt - 1);

figure(4)
plot([1:1:length(c_prime_post)]*dt*12,log(c_prime_post/mean(c_prime_pre) ))
xlabel('months'), ylabel('log c')
%vline((inde_time_shock-1)*dt,'r:','Aggregate Shock'),
title('log C response to shock')
grid('on')


%  figure for the multi-product paper appendix

%  Load data from the IRF fr n=1 in the multiproduct paper -- we must use the
%  same parameters here: Na=1 ,   Std(Dp)=0.1 ; B=20 ;  sigma=0.1 ;  
out_1       =  load(['output_n1_nfirms500K_using_p.mat']);
cpi1_dt     =  out_1.Cum_cpi;
Tmax_months = 24 ;
dt1          =  1/365/2  ;  %  simulation used 1/2 day 
t_vec       = [1:1:Tmax_months/12/dt1];  %  
figure(51)
plot(t_vec*dt1*12,cpi1_dt(t_vec))


figure(5)

plot([1:1:length(P_prime_post)]*dt*12,100*(dM+log(P_prime_post/mean(P_prime_pre) )),'r-',...
    t_vec*dt1*12,100*cpi1_dt(t_vec),'b--',  'LineWidth',2)
xlabel('Months elapsed after the shock ','FontSize',14,'Interpreter','Latex ')
ylabel('Aggregate Price Index','FontSize',14,'Interpreter','Latex ')
lh=legend('Exact solution $ \ \ \ \ $', 'Approximate solution $ \ \ \ \ $',0);
set(lh,'FontSize',16,'Interpreter','Latex ' )
grid('on')
axis([0 25 0 1.05])

%vline((inde_time_shock-1)*dt,'r:','Aggregate Shock'),
% title('$\log(P)$ after the shock','FontSize',16,'Interpreter','Latex ')
 
% orient landscape
% print -depsc check_irf_n1_dt_halfday_delta02_short
