% This code computes the IRF  using a non-quadratic return function for a  menu cost model as
% in Golosov and Lucas  with n=1;  the code uses the optimal (not the steady state) decision 
% rules  along the transition to compute the irf
% written by F. Lippi   2013 
clear all
close all

%parameters for impulse response
N_sim      = 50000  ;    % original:  300000 ; % 1e6/2; %number of firms simulated to obtain the invariant distribution
dt         = 1/12   ;    % 1/52; % 1/365/2   ;  % half a day, as in the  paper %0.01;
dt_sim     = 1/12   ;    % original: 1/365/2  ;  % length of time interval for the simulation
dM         = 0.10   ;    % percentage increase in money supply
time_shock = 1*1/12 ;    % time of the shock in the simulation of the IRF (in years)
length_irf = 2 ;         % length irf in years
T_end_irf  = time_shock + length_irf;   %  length of the response to the aggregate shock in years

% More params
readparams;

% Impulse responses
Total_time = 0:dt:T_end_irf; % in years
inde_time_shock = round(time_shock/dt) ; % in model periods

% compute the exact steady state rules (uses a subroutine)
Solve_SS;

filename=['results_policy_ss'];
save (filename)

% Find the invariant distribution (uses a subroutine)
disp('   ') 
disp(' computing p-gaps corresponding to invariant distribution ') 
disp('   ') 
Get_invariant ;

% save Vector of St St gaps p: V_p   
save invariant_gaps_drift V_p

%  use invariant distribution of gaps to compute St St consumption and price level
c_ss =  (alpha*eta/(eta-1))^(-1/epsilon) * (phi_0'*exp((1-eta)*V_p))^(1/(epsilon*(eta-1))) ;
P_ss =  ( 1/alpha )*(c_ss).^(-epsilon)  ;

disp('   ') 
disp(' Starting simulation for IRF')
disp('   ') 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Load data from the IRF fr n=1 in the multiproduct paper -- we must use the
%  same parameters here: Na=1 ,   Std(Dp)=0.1 ; B=20 ;  sigma=0.1 ;
out_1       =  load(['output_n1_nfirms500K_using_p.mat']);
cpi1_dt     =  out_1.Cum_cpi;
Tmax_months =  30 ;
dt1         =  1/365/2  ;  %  in fraction of years ; original simulation used 1/365/2 (halfday)
t_vec       =  [1:1:Tmax_months/12/dt1];  %  in model periods

%  Make a guess for the path of c_prime using the aprox model (uses expression form GE appendix)
c_prime_cdt =  c_ss*(  (1+cpi1_dt(t_vec)-0.01)).^(-1/epsilon) ;

% spline consumption  over clock in years SINCE the shock  (first obs is at t=0)
PP_cons_apx=spline((t_vec-1)*dt1,c_prime_cdt);
PP_cons=PP_cons_apx;

dates_irf = [time_shock:dt:T_end_irf ];% calendar time in years
c_agg_irf = ppval(dates_irf -time_shock ,PP_cons);

%  Compute expected change of consumption next period
for i=1:length(dates_irf)-1
    c_ratio(i) = ppval(dates_irf(i+1),PP_cons)/ppval(dates_irf(i),PP_cons);
end
c_ratio(length(dates_irf))=1;

%  simulation for IRF
Tn         = length(dates_irf);
T_vec      = [inde_time_shock:1:Tn] ; %   indices of IRF time in model periods


% display results
n_sim_disp=10000;
n_sim_disp_step=10000;

% Now , give c_prime_cdt, you must calculate  a decision triplet for each t in Total_time

%guess value functions at each node tn
for ti=1:Tn
    tn = Tn-ti+1;
    vhat_n(tn)        = vhat_ss;
    vbar_n(tn,:)      = vbar_ss;
    phat_n(tn)        = phat_ss;
end

n_it    = 1;
crit_c  = 1;
crit_val= 0.001/3; %0.001/2; % this criterion is measured as a fraction of the steady state consumption i.e. 0.01 is 1% error


Tn_sim = round(Tn*dt/dt_sim); % length of IRF  in model periods    
sum_c      = zeros(Tn_sim,1);
indx_time_shock_sim=round(dt/dt_sim*inde_time_shock);

while crit_c > crit_val && n_it < 20,
    
    warning off
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % FIND decision rules at other nodes recursively starting from the IRF end
    for ti = 2:Tn
        %          tic
        tn = Tn - ti+1;   %node of evaluation; tn=n is the last node, tn=1 is the first node, i.e. when the shock happens
        tauhat_tn=[];
        vhat_tn  =[];
        
        %   value function  in t+1
        %   This is the spline of vbar, the max between vbar and vhat is taken inside the va fct
        PP_v = spline(p_g,vbar_n(tn+1,:)) ;
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Compute vhat_t (vf with adjustment)
        LB=-.05;UB=0.05;
        [p_hat_t,vhat_neg] = fmincon(@(pp) -fct_vhat_irf(dt,PP_v,vhat_n(tn+1),pp,c_ratio(tn)),phat_ss,[],[],[],[],LB,UB,[],optimset(options,'TolX',1e-005','TolFun',1e-005,'Display','off'));
        vhat_t=-vhat_neg;
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Compute vbar_t (vf with no adjustment)
        for jj=1:length(p_g)
            pj=p_g(jj);
            vbar_t(1,jj) = fct_vbar_irf(dt,PP_v,vhat_n(tn+1),pj,c_ratio(tn)) ;
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %save value functions and policies
        vhat_n(tn)     = vhat_t;
        phat_n(tn)     = p_hat_t;
        vbar_n(tn,:)   = vbar_t ;
        
        % compute thresholds
        PP_b = spline(p_g,vbar_t) ;
        p_g_fine = min(p_g):0.001:max(p_g);
        vdif_n=[];  vdif_ss=[];
        for i=1:length(p_g_fine),
            vdif_ss(i)   = ppval(PP_b,p_g_fine(i)) - vhat_t;
        end
        vdif = vdif_ss; % if positive then inaction is optimal
        ilow = find(vdif>0,1);
        ilow      = max(1,ilow);
        low_p  = p_g_fine(ilow);
        ihigh= find(vdif>0, 1, 'last' );
        ihigh     = min(length(p_g_fine),ihigh);
        high_p =  p_g_fine(ihigh);
        %  now save thresholds
        plow_n(tn)  = low_p;
        phigh_n(tn) = high_p;
        
        display(['optimal policy chosen for t = ', num2str(tn)])
        
        vhat_t=[]; vbar_t=[]; p_hat_t=[];
        
        %        toc
    end % for ti
    
    % impose steady state thresholds at t = Tn
    plow_n(Tn)   = low_p_ss;
    phigh_n(Tn)  = high_p_ss;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %  SIMULATE IRF for given decision rules
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    for i=1:N_sim
        
        if i==n_sim_disp
            
            disp(['simulation for firm i = ',num2str(i)]);
            
            n_sim_disp= i + n_sim_disp_step;
            
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Object of interest
        p_tilde_t   = zeros(Tn_sim,1); % log-price gap of a given firm at each point in time, after eventual replacement/adjustment
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % First period
        time      = 0; % clock time (in years) since beginning of simulation
        inde_time = 0; % time index in model periods
        p_tilde   = V_p(i);  % initial price gap  for firm i , computed by " Get_invariant "
        
        while inde_time<Tn_sim
            time = time + dt_sim;  % clock
            inde_time = inde_time+1; % model periods
            
            % check if firm dies, in which case do forced observation/adjustment
            omega=rand;
            
            %CASE A:Replacement
            if omega>=exp(-delta*dt_sim)  %case of forced observation/adjustment
                p_tilde   = phat_ss;  %initial price gap
                
            else
                
                s = randn;
                p_tilde   =  p_tilde  + (gamma-mu)*dt_sim + sigma*sqrt(dt_sim)*s;
                %        Dp = 0 ;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % aggregate shock
                if inde_time == indx_time_shock_sim 
                    p_tilde = p_tilde-dM;
                end
                
                % Check if Price Adjust
                
                % map the simulation period inde_time into IRF  periods
                % to select the appropriate  decision rule
                
                rule_index = max(1,round(inde_time*dt_sim/dt));
                if (p_tilde<= plow_n(rule_index) ) || (p_tilde>= phigh_n(rule_index ) )
                    
                    p_tilde  = phat_n(rule_index);
                    
                end
                
            end
            
            %save
            p_tilde_t(inde_time)= p_tilde; % update price gap
            
            
        end % loop over time for firm "i"
        
        % update aggregate consumption adding the contribution of firm i
        sum_c    = sum_c  + 1/N_sim * (alpha*eta/(eta-1) *exp((p_tilde_t))).^(1-eta);
        
    end % loop over firms
    
    
    c_agg_irf_new_sim =  sum_c'.^(1/(epsilon*(eta-1)));
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Evaluate initial guess for consumption, save results
    
    
    %  map the consumption profile back into the original time scale
    PP_cons=spline([0:dt_sim:length_irf],c_agg_irf_new_sim(indx_time_shock_sim:end));
    
    %  now write cons in terms of original time scale
    c_agg_irf_new = ppval(dates_irf - time_shock ,PP_cons);
    
    crit_c = norm(c_agg_irf_new - c_agg_irf )/sqrt(Tn)/c_ss;
    
    disp(['finished simulation #', num2str(n_it),' iteration criterion c ',num2str(crit_c)])
    
    % update c-ratio
    
    for i=1:length(dates_irf)-1
        c_ratio(i) = ppval(dates_irf(i+1),PP_cons)/ppval(dates_irf(i),PP_cons);
    end
    c_ratio(length(dates_irf))=1;
    
    %%%%%%%%%%%%%%%
    
    c_agg_irf=c_agg_irf_new;
    
    % reset indices
    sum_c      = zeros(Tn_sim,1);
    n_sim_disp = n_sim_disp_step;
    
    n_it   = n_it+1;
    
end

T_irf=[1:1:length(T_vec)]   ;
% T_vec  is the indices of model observations that correspond to the irf

%consumption
% close
figure(1)
c_agg_irf_n1 = ppval((T_irf-1)*dt,PP_cons_apx); 
plot(T_irf*dt*12,100*(c_agg_irf_n1/c_ss-1),'b:',T_irf*dt*12,100*(c_agg_irf(T_vec)/c_ss-1),'r') 
lh=legend(' approx ', ' GE exact ',0);
hline([0])    
xlabel('months ','FontSize',14,'Interpreter','Latex ')

%%%%%%%%%%%%%%%%%%%%%%%%%%%

%  price level
P_GE  =  ( 1/alpha )*(c_agg_irf(T_vec)).^(-epsilon);
P_apx =  ( 1/alpha )*(c_agg_irf_n1).^(-epsilon);


figure(2)
plot(T_vec,100*(P_apx/P_ss-1+dM),'b:',T_vec,100*(P_GE/P_ss-1+dM),'r','LineWidth',2) 
lh=legend(' approx ', ' GE exact',0);
set(lh,'FontSize',16,'Interpreter','Latex ' )
xlabel('Months elapsed after the shock ','FontSize',14,'Interpreter','Latex ')
ylabel('Price level response  in \%','FontSize',14,'Interpreter','Latex ')
grid('on')
lh=hline(0.5,'g:');set(lh,'LineWidth',2) 
axis([0 25  0 1.1 ])
% 
% orient landscape
% print -depsc GE_on_price_level_d01
%  
%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(3)
plot(plow_n,[1:1:Tn],'b--', phat_n,[1:1:Tn],'r-.', phigh_n,[1:1:Tn],'b-','LineWidth',2) 
ylabel('Months elapsed after the shock ','FontSize',14,'Interpreter','Latex ')
xlabel('Decision thresholds ','FontSize',14,'Interpreter','Latex ')
lh=legend(' $\underline p$ $ \ \ \ \ $',' $\hat p$ $ \ \ \ \ $',' $\bar p$ $ \ \ \ \ $',0);
set(lh,'FontSize',16,'Interpreter','Latex ' )
grid('on')

% orient landscape
% print -depsc GE_on_decision_rules_d01

figure(4)
hist(V_p,200); vline(0)
xlabel('Price gaps','FontSize',14,'Interpreter','Latex ')
ylabel('frequency','FontSize',14,'Interpreter','Latex ')
 

