%  Solves the system of 5 equations for the Menu cost model with quadratic returns 
%  (but without approximating the value function);  Output is policy rule and vf:   p_dw,phat,p_up , C1, C2
%  The 5 equation system described in the notes is recursive;  it can be
%  split in the solution of the 3 equations for phat, phigh, plow, which imply C1 and C2 ; 
function [p_dw,phat,p_up,z1,z2,CC] = fct_solve_MC(B,rho,sigma,mu,psi)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Guess  policy using model without inflation
phat=0;
% VF for pure menu cost model wo drift
pbar_menu = (6*(sigma^2)*(psi)/B)^(1/4);
zeta_0   =  sqrt(2*rho/sigma^2) ;   
C_0 =   (B/rho*pbar_menu^2-psi)/ (2- exp(zeta_0*pbar_menu) - exp(-zeta_0*pbar_menu) )  ;
V_0 = B/rho^2 * sigma^2 + 2*C_0 ;
p_up= pbar_menu*(1+mu);
p_dw= -pbar_menu*(1+mu);

% Now compute roots of characteristic equation for probelm with inflation (mu>0)
z1 =  (mu + sqrt(mu^2+2*sigma^2*rho))/sigma^2;
z2 =  (mu - sqrt(mu^2+2*sigma^2*rho))/sigma^2;

%  Now solve 3 eq system   (these equations are the zeroes of the 2 smooth pasting conditions  and
%  the optimal return point, see the notes on the solution of the quadratic problem) 
options = optimset('MaxFunEvals',1000000,'MaxIter',10000000,'Display','off','TolX',1e-8);
warning off;
X0=[p_dw,phat,p_up ];

[x_star,SYSzero,flagSYS] = fsolve( @(x) fct_sys3(x,B,rho,psi,mu,z1,z2),[X0],options)     ;

p_dw = x_star(1) ; phat = x_star(2); p_up= x_star(3);

% compute Constants C1, C2   (in vec CC);  this solves the 2 value matching eqts

A_mat(1,1) = exp(z1*phat)-exp(z1*p_up);  
A_mat(1,2) = exp(z2*phat)-exp(z2*p_up);  
A_mat(2,1) = exp(z1*phat)-exp(z1*p_dw);  
A_mat(2,2) = exp(z2*phat)-exp(z2*p_dw);  
BB(1,1)  = B/rho * (p_up^2 - phat^2 - 2*mu/rho*(p_up-phat) ) - psi ; 
BB(2,1)  = B/rho * (p_dw^2 - phat^2 - 2*mu/rho*(p_dw-phat) ) - psi ; 
CC =A_mat\BB; 
   
