% this code computes Ap_n by simulation where
% Let x= (x_1,x_2,....,x_n) be uniformly distributed in the n dimensional
% sphere, so that || x|| = 1.
% Ap_n = E[ (1/n)(x_1+x_2+....+x_n) | x_1+x_2,....+x_n >0 ] .

clear all;
close all;


tic

nn = 30; % maximum number of dimensions 

% we generate x uniformly distributed in the n-dim spehere by
% taking z Norm(0, I_n) and letting x = z / ||z|| . 

nsim =1000000; % number of simulations

z = randn(nsim,nn);

for i=1:nsim;
    for n=1:nn
        z_i = z(i,1:n);
        avg_zi = mean(z_i);
        if avg_zi < 0
            Ap_n(i,n) = - avg_zi/norm(z_i);
        else    
            Ap_n(i,n) =   avg_zi/norm(z_i);
        end
    end
end


Ap_n_avg = mean(Ap_n(:,1:nn));
Ap_n_ste = std(Ap_n(:,1:nn))./sqrt(nsim);

diary_name_A =['Ap_n_nn',num2str(nn),'nsim',num2str(nsim/1000,3),'K'];

diary diary_name_A

display(' ')
display(['number of products n from 1 to ', num2str(nn)])
display(' ');
display(['based on a sample of size ', num2str(nsim)]);
display(' ');
display('n , Mean(Ap_n),Stde(Ap_n)');
display(num2str([[1:1:nn]',Ap_n_avg',Ap_n_ste'],3))

% z = randn(nsim,n);
% 
% for i=1:nsim;
%     z_i = z(i,1:n);
%     sum_zi =sum(z_i);
%     if sum_zi<0
%         Ap_n(i) = - sum_zi/norm(z_i);
%     else    
%         Ap_n(i) =   sum_zi/norm(z_i);
%     end
% end

diary off

toc

figure(1);
plot([1:1:nn],Ap_n_avg,'k-',[1:1:nn],Ap_n_avg+Ap_n_ste,'r.',[1:1:nn],Ap_n_avg-Ap_n_ste,'r.');
title('E[x_1+x_2+...+x_n | x_1+x_2+...+x_n>0] where x=z/norm(z), z~N(0,I_n) ');
xlabel('number of dimensions n')
ylabel('simulated mean, and 1 ste band')
text(nn*.8,0.9,['nsim = ',num2str(nsim/1000,4), ' K']);

