% This files simulates the process for y and z of firms in the multiproduct
% model with correlation to produce the IRF after a monetary shock
%
% it takes as inputs:
% delta : size of monteary shock
% n : number of products
% innovation volatitlies sigma and bar_sigam
% boundary of range of innaction, which can be comptudes using the
% paramgers of the value function (a polinomial on y and z), fit.beta
%
% we read all these parameters from a structured created when we solved
% numerically for the value function, in out_nX_corr_XXX

clear all; close all;

delta=0.01;

output=load('out_n2_corr_0.mat');

n = output.n;
sigma = output.sigma;
bar_sigma = output.sigmabar;
beta = output.beta;
D = output.D;
ybar_zzero = output.ybar_zzero;

T = 1; % length of time to compute initial conditoins for the IRF, in years
% during this peeriod we use symmetry to cheaply get a draw from the
% invariant distribution

Nfirms=40000; 

Nfirms = 2*round(Nfirms/2); % makes sure that Nfirms is an even number.

yp=zeros(1,Nfirms/2);
zp=zeros(1,Nfirms/2);

y_reflected =zeros(1,Nfirms);
z_reflected =zeros(1,Nfirms);

y=zeros(1,Nfirms/2);
z=zeros(1,Nfirms/2);

y_avg = zeros(1,round(T/D));
y_std = zeros(1,round(T/D));

z_avg = zeros(1,round(T/D));
z_std = zeros(1,round(T/D));

fa = zeros(1,round(T/D));

y_avg_adj = zeros(1,round(T/D));
z_avg_adj = zeros(1,round(T/D));

y_std_adj = zeros(1,round(T/D));
z_std_adj = zeros(1,round(T/D));

for t=1:round(T/D);
    
    y_avg(t) = mean(y_reflected);
    y_std(t) = std(y_reflected);
    y_3rd(t) = mean(y_reflected.^3);
    y_4rd(t) = mean(y_reflected.^4);
    z_avg(t) = mean(z_reflected);
    z_std(t) = std(z_reflected);
    
    % draw innovations for all firms corresponding to transition from t
    % to t+1
    e_a = rand(1,Nfirms/2);
    e_b = rand(1,Nfirms/2);
    e_c = rand(1,Nfirms/2);
    
    % initialize number of firms that change prices
    fa(t)=0;
    
    for i=1:Nfirms/2
        % update state of the firm from the previous period
        y(i) = yp(i);
        z(i) = zp(i);
        
        %convert uniform draws into binomial:
        
        if e_a(i) < 1/2
            e_a(i) = -1;
        else
            e_a(i) = 1;
        end
        if e_b(i) < 1/2
            e_b(i) = -1;
        else
            e_b(i) = 1;
        end
        if e_c(i) < 1/2
            e_c(i) = -1;
        else
            e_c(i) = 1;
        end
        
        % check whether price change or not given current state:
        if y(i) <= fun_ybar_adjusted(z(i),beta,n,ybar_zzero) ;
           
            % if no price change use innovation to update
            [ yp(i) zp(i)] = fun_law_motion_state(y(i),z(i),e_a(i),e_b(i),e_c(i),D,sigma,bar_sigma,n);
            
        else
            % after price change set state to zero gap:
            yp(i) = 0;
            zp(i) = 0;
            % update number of firms that change price and value of y and z
            % for those that do change:
            fa(t) = fa(t)+1;
            y_adj(fa(t)) = y(i);
            z_adj(fa(t)) = z(i);
            
        end
        
    end
    
    % extend result for firm i to mirro imagine firm i+Nfirm,
    % reflecting its z value and keeping the same y.
        
    y_reflected =   [ yp ,  yp  ];
    z_reflected =   [ zp , -zp ];
    
    
    % compute average and standard deviation of state for firm that change
    % prices
    if fa(t) > 0
        
        % extend adj to mirror image firms, reflecting z shocks
        y_adj_reflected = [ y_adj,  y_adj ] ;
        z_adj_reflected = [ z_adj, -z_adj ] ;
        
        y_avg_adj(t) = mean(y_adj_reflected);
        z_avg_adj(t) = mean(z_adj_reflected);
        y_std_adj(t) = std(y_adj_reflected);
        z_std_adj(t) = std(z_adj_reflected);
    end
    
end

fa=fa*2; % count the number of adjustment across the Nfirms 

time=[1:1:round(T/D)];
figure(1); plot(time,y_avg,time,z_avg);
legend('E(y)','E(z)');

figure(2); plot(time,y_std,time,z_std);
legend('Std(y)','Std(z)');


figure(3); plot(time,z_std_adj)
legend('Std(z)  for firms that adjust');


% compute some statistics on the second half of the sample to check if the
% distribution has converged to steady state

% average number of adjustment per year
avg_fa = mean(fa(round(T/D/2):end)/Nfirms);
Na = avg_fa/D;

std_z_adj = mean(z_std_adj(round(T/D/2):end));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%  Now compute shock the steady state and compute the impulse response

%  y_reflected and z_reflected are our sample of the steady state

%  yp and zp one instant after shock 
zp = z_reflected - n*delta ;
yp = y_reflected - 2*delta*z_reflected + n*delta^ 2 ;


Ti=2;  % number of years for the impulse response

% vector of number of firms that adjust price at each time s
Nadj = zeros(1,round(Ti/D));

% vector with the sum of he price changes of firms that adjust at each time s
DeltaP = zeros(1,round(Ti/D));


for i=1:Nfirms
        
    % initial value of the IRF for firm i after the monetary shock
    yy = yp(i);
    zz = zp(i);
    
    s=1; % initialize time s=1 as the time of the monetary shock
    
    while s <= round(Ti/D);
        
        % check whether price change or not given current state:
      
        if yy <= fun_ybar_adjusted(zz,beta,n,ybar_zzero) ;
            
            %convert uniform draws into binomial:
            ee_a = rand;
            ee_b = rand;
            ee_c = rand;
            
            if ee_a < 1/2
                ee_a = -1;
            else
                ee_a = 1;
            end
            if ee_b < 1/2
                ee_b = -1;
            else
                ee_b = 1;
            end
            if ee_c < 1/2
                ee_c = -1;
            else
                ee_c = 1;
            end
            
            % if no price change use innovation to update
            [ yy zz] = fun_law_motion_state(yy,zz,ee_a,ee_b,ee_c,D,sigma,bar_sigma,n);
            
            %yy=yyy;
            %zz=zzz;
            
            s=s+1;
            
        else
            % firm changes price, so keep track of # of firms and cumulated price changn
            Nadj(s) = Nadj(s)+1;
            DeltaP(s) = (-zz/n)/(Nfirms) + DeltaP(s) ;
            s = round(Ti/D) + 10; % 
        end
        
    end
    % check if no price change has happend at time Ti
    if s == round(Ti/D)+1 % forced price adjustment at time Ti
            Nadj(round(Ti/D)) = Nadj(round(Ti/D)) + 1;
            DeltaP(round(Ti/D)) = -zz/n/(Nfirms) + DeltaP(round(Ti/D));
    end
end

% P_agg(s) is the IRF to the price level s model periods (each of length D) after the monetary shock
% of size delta in period s=1. 
P_agg(1)=DeltaP(1);
for s=2:round(Ti/D)

    P_agg(s) = P_agg(s-1) + DeltaP(s);    
    
end

corr= bar_sigma^ 2 / (sigma^ 2 + bar_sigma^ 2) ; 

figure(4); plot([0:D: Ti-D] , P_agg);
ti=title({'Impulse response',...
         ['$n =$ ', num2str(n,2), ', correlation($dp_i,dp_j$) = ',num2str(corr,3)]});
set(ti,'Interpreter','Latex','Fontsize',16);
xl=xlabel('Time elapsed since the shock');
yl=ylabel('Aggregate Price ');
set(xl,'Interpreter','Latex','Fontsize',16);
set(yl,'Interpreter','Latex','Fontsize',16);
hline(delta, 'r' );


fname = ['IRF_n', num2str(n,2),'_corr_', num2str(corr,3) ,'.mat'];

IRF.n       = n;
IRF.sigma   = sigma;
IRF.sigmabar= bar_sigma;
IRF.D       = D;  % length of time period
IRF.Na      = Na; % at steady state
IRF.stdzadj = std_z_adj; % at steady state
IRF.Nfirms  = Nfirms; % number of firms used in simulations (cross section)
IRF.T       =  T ; % years used to compute ss
IRF.delta   = delta;  % size of monetary shock
IRF.Ti      =  Ti ; % years used to compute IRF
IRF.Pagg    = P_agg; % cumulative IRF price level
IRF.Nadj    = Nadj;  % number of firms that adjusted s model periods after monetary shock
 
save(fname, '-struct', 'IRF');





