%  this file uses the output from the simulation of the price changes to
%  draw some IRF functions  with and without correlation

clear all; close all; 

% load the IRF  data

% the plots are standardized to these stats

delta=.01;  % size of monetary shock
Na=1;  % number of adjustment per year
std_Dp = 0.1; %  std of price adjustments
capT = 18; % months to be plotted 


irf_n1_0=load('IRF_normal_n1_corr_0_Nfirms500K.mat'); 

irf_n2_0 =load('IRF_normal_n2_corr_0_Nfirms250K.mat'); 
irf_n2_05=load('IRF_normal_n2_corr_0.5_Nfirms250K.mat');

irf_n3_0=load('IRF_normal_n3_corr_0_Nfirms250K.mat'); 
irf_n3_05=load('IRF_normal_n3_corr_0.5_Nfirms250K.mat');

irf_n10_0=load('IRF_normal_n10_corr_0_Nfirms250K.mat'); 
irf_n10_05=load('IRF_normal_n10_corr_0.5_Nfirms250K.mat');

irf_n30_0=load('IRF_normal_n30_corr_0_Nfirms250K.mat');
irf_n30_05=load('IRF_normal_n30_corr_0.5_Nfirms250K.mat');
 
irf_n50_0=load('IRF_normal_n50_corr_0_Nfirms250K.mat'); 
irf_n50_05=load('IRF_normal_n50_corr_0.5_Nfirms250K.mat');

D = irf_n2_0.D;
Ti = irf_n2_0.Ti;
time_months=[D:D:Ti]*12; 


%  trim time for figure
indx_fig = [1:1: find(time_months<capT, 1, 'last' )];
time=time_months(indx_fig);


%  load output from the codes for no correlation
%out_1 = load(['output_n1_nocorr.mat']);
% cpi1_dt=out_1.Cum_cpi;
%t_year=[1:1:length(cpi1_dt)]/52/irf_n2_0.Na;

%  all P   adjusted to correspond to a shock os delta/std(Dp) = 0.1   and  Na=1  and Std(Dp)=1 

%  Homogeneity adjustments for n1
time1        = irf_n1_0.Na*time;
sim_n1_r0    = load('simulp_n1_corr_0.mat');
stdDp_n1     = sim_n1_r0.StdDp;
p_n1_r0      = irf_n1_0.Pagg(indx_fig)*std_Dp/stdDp_n1 *100; %*.1/stdDp_n1 ; % so that delta/std(dp)=.1 

%  Homogeneity adjustments for n2 corr 0
time2_0      = irf_n2_0.Na*time;
sim_n2_r0    = load('simulp_n2_corr_0.mat'); 
stdDp_n2_0   = sim_n2_r0.StdDp;
p_n2_r0      = irf_n2_0.Pagg(indx_fig)*std_Dp/stdDp_n2_0 *100;

%  Homogeneity adjustments for n2 corr 0.5
time2_05    = irf_n2_05.Na*time;
sim_n2_r05   = load('simulp_n2_corr_0.5.mat'); 
stdDp_n2_05 = sim_n2_r05.StdDp;
p_n2_r05     = irf_n2_05.Pagg(indx_fig)*std_Dp/stdDp_n2_05 *100;



%%%%%%%%%%%%%%%%%%%


figure(1); 
ph=plot(time2_0,p_n2_r0,'b',time2_05,p_n2_r05,'k--',time1,p_n1_r0,'r:');
set(ph,'Linewidth',4);

lh=legend('$n=2 , \rho = 0$', '$n=2, \rho = 0.5 \ \ $', '$n=1$', 0); 
set(lh,'Interpreter','Latex','Fontsize',16);
th=title(' Impulse response for $n=2$'); 
set(th,'Interpreter','Latex','Fontsize',16);
yh=ylabel('Aggregate price level (in \%)'); 
set(yh,'Interpreter','Latex','Fontsize',16);
xh=xlabel('Months after the shock '); 
set(xh,'Interpreter','Latex','Fontsize',16);
axis([0  20  0 1.05 ]);
 
%orient landscape
print ('-depsc','irf_corr_n2');



%%%

%%% case n=3

%  Homogeneity adjustments for n3 corr 0
time3_0      = irf_n3_0.Na*time;
sim_n3_r0    = load('simulp_n3_corr_0.mat'); 
stdDp_n3_0   = sim_n3_r0.StdDp;
p_n3_r0      = irf_n3_0.Pagg(indx_fig)*std_Dp/stdDp_n3_0 *100;

%  Homogeneity adjustments for n3 corr 0.5
time3_05    = irf_n3_05.Na*time;
sim_n3_r05   = load('simulp_n3_corr_0.5.mat'); 
stdDp_n3_05 = sim_n3_r05.StdDp;
p_n3_r05     = irf_n3_05.Pagg(indx_fig)*std_Dp/stdDp_n3_05 *100;


figure(2); 
ph=plot(time3_0,p_n3_r0,'b',time3_05,p_n3_r05,'k--',time1,p_n1_r0,'r:');
set(ph,'Linewidth',4);

lh=legend('$n=3 , \rho = 0$', '$n=3, \rho = 0.5 \ \ $', '$n=1 ,  \ $', 0); 
set(lh,'Interpreter','Latex','Fontsize',16);
th=title(' Impulse response for $n=3$'); 
set(th,'Interpreter','Latex','Fontsize',16);
yh=ylabel('Aggregate price level (in \%)'); 
set(yh,'Interpreter','Latex','Fontsize',16);
xh=xlabel('Months after the shock '); 
set(xh,'Interpreter','Latex','Fontsize',16);
axis([0  20  0 1.05 ]);

 
%orient landscape
print ('-depsc','irf_corr_n3');

%  case n=10

%  Homogeneity adjustments for n10 corr 0
time10_0      = irf_n10_0.Na*time;
sim_n10_r0    = load('simulp_n10_corr_0.mat'); 
stdDp_n10_0   = sim_n10_r0.StdDp;
p_n10_r0      = irf_n10_0.Pagg(indx_fig)*std_Dp/stdDp_n10_0 *100;

%  Homogeneity adjustments for n10 corr 0.5
time10_05     = irf_n10_05.Na*time;
sim_n10_r05   = load('simulp_n10_corr_0.5.mat'); 
stdDp_n10_05  = sim_n10_r05.StdDp;
p_n10_r05     = irf_n10_05.Pagg(indx_fig)*std_Dp/stdDp_n10_05 *100;


%%%

figure(3); 
ph=plot(time10_0,p_n10_r0,'b',time10_05,p_n10_r05,'k--',time1,p_n1_r0,'r:');
set(ph,'Linewidth',4);

lh=legend('$n=10 , \rho = 0$', '$n=10, \rho = 0.5 \ \ $', '$n=1   \ $', 0); 
set(lh,'Interpreter','Latex','Fontsize',16);
th=title(' Impulse response for $n=10$'); 
set(th,'Interpreter','Latex','Fontsize',16);
yh=ylabel('Aggregate price level (in \%)'); 
set(yh,'Interpreter','Latex','Fontsize',16);
xh=xlabel('Months after the shock '); 
set(xh,'Interpreter','Latex','Fontsize',16);
axis([0  20  0 1.05 ]);

 
%orient landscape
print ('-depsc','irf_corr_n10');


%%%

%  case n=30

%  Homogeneity adjustments for n30 corr 0
time30_0      = irf_n30_0.Na*time;
sim_n30_r0    = load('simulp_n30_corr_0.mat'); 
stdDp_n30_0   = sim_n30_r0.StdDp;
p_n30_r0      = irf_n30_0.Pagg(indx_fig)*std_Dp/stdDp_n30_0 *100;

%  Homogeneity adjustments for n30 corr 0.5
time30_05     = irf_n30_05.Na*time;
sim_n30_r05   = load('simulp_n30_corr_0.5.mat'); 
stdDp_n30_05  = sim_n30_r05.StdDp;
p_n30_r05     = irf_n30_05.Pagg(indx_fig)*std_Dp/stdDp_n30_05 *100;


figure(4); 
ph=plot(time30_0,p_n30_r0,'b',time30_05,p_n30_r05,'k--',time1,p_n1_r0,'r:');
set(ph,'Linewidth',4);

lh=legend('$n=30 , \rho = 0$', '$n=30, \rho = 1/2 \ \ $', '$n=1  $', 0); 
set(lh,'Interpreter','Latex','Fontsize',16);
th=title(' Impulse response for $n=30$'); 
set(th,'Interpreter','Latex','Fontsize',16);
yh=ylabel('Aggregate price level '); 
set(yh,'Interpreter','Latex','Fontsize',16);
xh=xlabel('Months after the shock '); 
set(xh,'Interpreter','Latex','Fontsize',16);

axis([0  20  0 1.05 ]);

%orient landscape
print ('-depsc','irf_corr_n30');


%%%%%%%%%%%%%

%%%

%  case n=50

%  Homogeneity adjustments for n50 corr 0
time50_0      = irf_n50_0.Na*time;
sim_n50_r0    = load('simulp_n50_corr_0.mat'); 
stdDp_n50_0   = sim_n50_r0.StdDp;
p_n50_r0      = irf_n50_0.Pagg(indx_fig)*std_Dp/stdDp_n50_0 *100;

%  Homogeneity adjustments for n50 corr 0.5
time50_05     = irf_n50_05.Na*time;
sim_n50_r05   = load('simulp_n50_corr_0.5.mat'); 
stdDp_n50_05  = sim_n50_r05.StdDp;
p_n50_r05     = irf_n50_05.Pagg(indx_fig)*std_Dp/stdDp_n50_05 *100;


ftsize=18;

figure(5); 
ph=plot(time50_0,p_n50_r0,'b',time50_05,p_n50_r05,'k--',time1,p_n1_r0,'r:');
set(ph,'Linewidth',4);

lh=legend('$n=50 , \rho = 0$', '$n=50, \rho = 1/2 \ \ $', '$n=1  $', 0); 
set(lh,'Interpreter','Latex','Fontsize',ftsize);
th=title(' Impulse response for $n=50$'); 
set(th,'Interpreter','Latex','Fontsize',ftsize);
yh=ylabel('Aggregate price level '); 
set(yh,'Interpreter','Latex','Fontsize',ftsize);
xh=xlabel('Months after the shock '); 
set(xh,'Interpreter','Latex','Fontsize',ftsize);

axis([0  20  0 1.05 ]);

%orient landscape
print ('-depsc','irf_corr_n50');


%%%%%%%%%%%%%


%  Draw figures with IRFs   with no correlation

figure(6); 
ph=plot(time1,p_n1_r0,'b',time2_0,p_n2_r0,'k--',time3_0,p_n3_r0,'r:',time10_0,p_n10_r0,'g:');
set(ph,'Linewidth',4);

lh=legend('$n=1$', '$n=2$', '$n=3 $','$n=10 $', 0); 
set(lh,'Interpreter','Latex','Fontsize',ftsize);
th=title(' Impulse response for $\rho=0$ various $n$'); 
set(th,'Interpreter','Latex','Fontsize',ftsize);
yh=ylabel('Aggregate price level (in \%)'); 
set(yh,'Interpreter','Latex','Fontsize',ftsize);
xh=xlabel('Months after the shock '); 
set(xh,'Interpreter','Latex','Fontsize',ftsize);
axis([0  20  0 1.05 ]);
hline(delta*1*100,'k','');
hline(delta*0.5*100,'k','');

 
orient landscape
print ('-depsc','irf_nocorr_all_n');


%  Draw figures with IRFs   with correlation 1/2

%  this figure appears in the paper

figure(7); 
ph=plot(time1,p_n1_r0,'b',time2_05,p_n2_r05,'k--',time3_05,p_n3_r05,'r:',time10_05,p_n10_r05,'g:');
set(ph,'Linewidth',4);

lh=legend('$n=1$', '$n=2$', '$n=3 $','$n=10  \ \ \ $',0); 
set(lh,'Interpreter','Latex','Fontsize',16);
th=title(' Impulse response for $\rho=0.5$ various $n$'); 
set(th,'Interpreter','Latex','Fontsize',16);
yh=ylabel('Aggregate price level (in \%)'); 
set(yh,'Interpreter','Latex','Fontsize',16);
xh=xlabel('Months after the shock '); 
set(xh,'Interpreter','Latex','Fontsize',16);
%grid on;
hline(delta*1*100,'k','');
hline(delta*0.5*100,'k','');
axis([0  20  0 1.05 ]);

 
orient landscape
print ('-depsc','irf_corr05_all_n');

%%%%%%

%%% compares IRF of n=1 case obtained by simulation with analytical
%%% expression


% computue IRF for Na=1, std_Dpn and delta

sigma=0.1;
p_n1_r0_anal = fct_Pt_anal(time1/12,delta,std_Dp,sigma,30)*100;

figure(8); 
ph=plot(time1,p_n1_r0,'b',time1,p_n1_r0_anal,'k--');
set(ph,'Linewidth',4);

lh=legend('$simulated$', '$analytical$', 0); 
set(lh,'Interpreter','Latex','Fontsize',16);
th=title(' Impulse response for $n=1$'); 
set(th,'Interpreter','Latex','Fontsize',16);
yh=ylabel('Aggregate price level (in \%)'); 
set(yh,'Interpreter','Latex','Fontsize',16);
xh=xlabel('Months after the shock '); 
set(xh,'Interpreter','Latex','Fontsize',16);
axis([0  20  0 1.05 ]);

%orient landscape
print ('-depsc','irf_n1_simul_analytic');




