% this script uses the output from the solve_multip_corr.m to and plots the value function and optimal policy for the
% multiproduct firm's pricing problem allowing correlation between the price gaps

clear all; close all;

% set problem params
n=2; 
corr_dp =0; % 0.5;


%outfilename = ['out_n',num2str(n,2),'_corr_',num2str(corr_dp,3),'.mat']; 

outfilename = ['out_n',num2str(n,2),'_cross_corr_',num2str(corr_dp,3),'.mat']; 

load(outfilename);


% 
%%%%%% creates a new grid for graphs
i=0;
NNy=100;
bary_per =n*sqrt( psi*(sigma^2+sigmabar^2)*6/(B*n) );% ybar withe perfect correlation
bary     = sqrt(psi/B*(sigma^2+sigmabar^2)*2*(n+2)); % ybar with no correlation
YY_g = [0:bary*2/(NNy-1):bary*2];
zz_step = sqrt(bary*2/(NNy-1)*n);

for yi=1:NNy;
    y=YY_g(yi);
    zlow  = -sqrt(y*n);
    zhigh =  sqrt(y*n);
    NNz = max(1,round((zhigh-zlow)/zz_step));
    zzz=(zhigh-zlow)/NNz;
    for j=0:NNz
        i=i+1;
        ygg(i) = y;
        zgg(i) = zlow+j*zzz;
    end
end
NNN=length(ygg);
% 
maxv=max(vg);
% % value function on new grid
 vgraph= fun_eval_v(ygg,zgg,beta,psi,maxv);
%
%%%% computes the values of (y,z) where there is control (action)
maxv=max(vgraph);
j=1;
for i=1:NNN
    if vgraph(i) >= maxv - psi/1000000;
        control(j)   = maxv;
        ygcontrol(j) = ygg(i);
        zgcontrol(j) = zgg(i);
        j=j+1;
    end
end

%  find minimum value of ybar in the contour  of control region
indx_zzero =find(zgcontrol==0);
ybar_zzero = min(ygcontrol(indx_zzero) );


%%%%% plots value function %%%%%%

figure(1); plot3(ygg,zgg,vgraph,'.',ygcontrol,zgcontrol,control,'*','Linewidth',2);
ti=title('Value function $v(y,z)$');
set(ti,'Interpreter','Latex','Fontsize',16)
xl=xlabel('$y = \sum_i p_i^2$');
set(xl,'Interpreter','Latex','Fontsize',16)
yl=ylabel('$z = \sum_i p_i$');
set(yl,'Interpreter','Latex','Fontsize',16)
zl=zlabel('$v(y,z)$');
set(zl,'Interpreter','Latex','Fontsize',16)
axis([0 1.01*max(ygg) 1.01*min(zgg) 1.01*max(zgg) 0.999*min(vgraph) 1.001*max(vgraph)]);

%  flatten value function
maxv=max(vg);
for i=1:length(ygg);
    if fun_eval_dvdy(ygg(i),zgg(i), beta) < 0
        vgraph(i)=maxv;
    end
end

% 
j=1;
for i=1:length(vgraph);
    if (vgraph(i) > maxv - psi/100 ) &&  (fun_eval_dvdy(ygg(i),zgg(i), beta) <= 0) ...
            || ygg(i) > fun_ybar(zgg(i),beta,n)   ; 
        control(j)   = maxv;
        ygcontrol(j) = ygg(i);
        zgcontrol(j) = zgg(i);
        j=j+1;
    end
end

font_size=14;

figure(2); plot3(ygg,zgg,vgraph,'.',ygcontrol,zgcontrol,control,'*','Linewidth',2);
grid on;
% ti=title({'Value function $v(y,z)$',
% ['For $n = $', num2str(n,2), ' , shocks correlation is ', num2str(corr_dp,1),...
%     ' , $B = $', num2str(B,2),' , $\psi/n = $', num2str(psi/n,1) ]});
ti=title(['( $n = $', num2str(n,2), ' , shocks correlation is ', num2str(corr_dp,1),...
    ' , $B = $', num2str(B,2),', $D = $', num2str(C,2),' , $\psi/n = $', num2str(psi/n,1) ,' )' ]);
set(ti,'Interpreter','Latex','Fontsize',font_size)
xl=xlabel('$y = \sum_{i=1}^n \, p_i^2$');
set(xl,'Interpreter','Latex','Fontsize',font_size)
yl=ylabel('$z = \sum_{i=1}^n \, p_i$');
set(yl,'Interpreter','Latex','Fontsize',font_size)
zl=zlabel('$v(y,z)$');
set(zl,'Interpreter','Latex','Fontsize',font_size)
%axis([0 1.01*max(ygg) 1.01*min(zgg) 1.01*max(zgg) 0.999*min(vgraph) 1.001*max(vgraph)]);

% orient landscape
fname2 = ['fig_vfun_n_', num2str(n,2),'_corr_',num2str(corr_dp,2),'.eps'];
print ('-depsc',fname2);

%return 

nzval=30;
stdz=sqrt(n*sigma^2+n^2*sigmabar^2);
%stdz = sqrt(n*(.5*bary+.5*bary_per));
zgrid=[-4*stdz:stdz/nzval:stdz*4];
for j=1:length(zgrid);
    vgz(:,j) = fun_eval_v(YY_g,ones(1,length(YY_g))*zgrid(j),beta,psi,maxv);
end

%  compute boundary ybar(zeta)  based on derivative
for j=1:length(zgrid)
    ybar(j) = fun_ybar(zgrid(j),beta,n);
end


%%%% plots boundary of inaction based on making derivative w.r.t. y = 0 %%%%

figure(3); plot(zgrid,ybar,'k.',zgrid,zgrid.^2/n,'b-','Linewidth',2); %grid on;
% ti=title({'Boundary of range of inaction $\bar{y}(z)$, based on $v_y(\bar y,z)=0$',
%     ['For $n = $', num2str(n,2), ' , $\sigma = $', num2str(sigma,1), ' , $\bar{\sigma} = $', num2str(sigmabar,1),...
%     ' , $B = $', num2str(B,2),' , $\psi/n = $', num2str(psi/n,1) ]});
ti=title(['( $n = $', num2str(n,2), ' , shocks correlation is ', num2str(corr_dp,1),...
    ' , $B = $', num2str(B,2),', $D = $', num2str(C,2),' , $\psi/n = $', num2str(psi/n,1) ,' )' ]);set(ti,'Interpreter','Latex','Fontsize',14)
le=legend('$\bar{y}(z)$', '$z^2/n$','Location','NorthWest');
set(le,'Interpreter','Latex','Fontsize',font_size)
xl=xlabel('$z = \sum_{i=1}^n \, p_i$');
set(xl,'Interpreter','Latex','Fontsize',font_size)
yl=ylabel('$ y = \sum_{i=1}^n \, p_i^2$');
set(yl,'Interpreter','Latex','Fontsize',font_size)
hline(bary,'k-');
th1=text(min(zgrid),bary*1.05,'uncorrelated case');
set(th1,'Interpreter','Latex','Fontsize',12);
%hline(bary,'k-');
%th2=text(min(zgrid),bary_per*1.05,'correlation 1 case');
%set(th2,'Interpreter','Latex','Fontsize',12);

%hline(bary_per,'k-');

t1=text(min(zgrid)/4,bary*0.8,'Inaction Region');
set(t1,'Interpreter','Latex','Fontsize',font_size);

t2=text(min(zgrid)/4,bary_per*0.9,'Adjustment Region');
set(t2,'Interpreter','Latex','Fontsize',font_size);


% orient landscape
 fname3 = ['fig_ybar_n_', num2str(n,2),'cross_corr_',num2str(corr_dp,2),'.eps'];
 print ('-depsc',fname3);


% %%%% plots contour levels of value function  %%%%%
% maxy=max(min(ybar),max(bary,bary_per));
% [ZGG,YGG] = meshgrid(min(zgrid):0.001:max(zgrid),0:0.0001:maxy*1.10);
% si=size(ZGG);
% for i=1:si(1);
%     for j=1:si(2)
%         y=YGG(i,j);
%         z=ZGG(i,j);
%         if z^2/n < y
%             VGG(i,j) =  fun_eval_v(y,z,beta,psi,maxv);
%         else
%             VGG(i,j) =  fun_eval_v(0,0,beta,psi,maxv);
%         end
%     end
% end
% 
% lowv=min(vgraph);
% highv=max(vgraph);
% nlevels=50;
% levelsv=[lowv:(highv-lowv)/(nlevels-1):highv];
% 
% figure(4); contour(ZGG,YGG,VGG,levelsv);
% ti=title({'Contour plot of value function $v(y,z)$ in inaction region',
%     ['For $n = $', num2str(n,2), ' , $\sigma = $', num2str(sigma,2), ' , $\bar{\sigma} = $', num2str(sigmabar,2),...
%     ' , $B = $', num2str(B,2),' , $\psi = $', num2str(psi,2) ]});
% set(ti,'Interpreter','Latex','Fontsize',14)
% xl=xlabel('$z = \sum_i p_i$');
% set(xl,'Interpreter','Latex','Fontsize',16)
% yl=ylabel('$ y = \sum_i p_i^2$');
% set(yl,'Interpreter','Latex','Fontsize',16);
% hline(bary,'k-','uncorrelated case');
% hline(bary_per,'k-','correlation 1 case');
% 
% orient landscape
% fname4 = ['fig_contour_n_', num2str(n,2),'_corr_',num2str(corr,2),'.eps'];
% print ('-depsc' , fname4);


% %   compare boundaries ybar 
% 
% figure(5)
% 
% plot(zgrid,ybar,'k.',zgrid,zgrid.^2/n,'b-', zgcontrol,ygcontrol,'*','Linewidth',2);
% hline(bary,'k-','uncorrelated case');
% hline(bary_per,'k-','correlation 1 case'); 
% 
% 
% ti=title({'Boundary of range of inaction $\bar{y}(z)$, based on $v_y(\bar y,z)=0$',
%     ['For $n = $', num2str(n,2), ' , $\sigma = $', num2str(sigma,2), ' , $\bar{\sigma} = $', num2str(sigmabar,2),...
%     ' , $B = $', num2str(B,2),' , $\psi = $', num2str(psi,2) ]});
% set(ti,'Interpreter','Latex','Fontsize',14)
% le=legend('$\bar{y}(z)$', '$z^2/n$','Location','North');
% set(le,'Interpreter','Latex','Fontsize',16)
% xl=xlabel('$z = \sum_i p_i$');
% set(xl,'Interpreter','Latex','Fontsize',16)
% yl=ylabel('$\bar y(z)= \sum_i p_i^2$');
% set(yl,'Interpreter','Latex','Fontsize',16)
% hline(bary_per,'k-','correlation 1 case');

% orient landscape
% fname3 = ['fig_ybar_n_', num2str(n,2),'_corr_',num2str(corr,2),'.eps'];
% print ('-depsc',fname3);


