% This files computes the marginal distibution of price changes for the
% multiproduct problem with n firms and correlated innovations
%
% it takes as inputs:
% n : number of products
% innovation volatitlies sigma and bar_sigam
% boundary of range of innaction, which can be comptudes using the
% paramgers of the value function (a polinomial on y and z), fit.beta
%
% we read all these parameters from a structured created when we solved
% numerically for the value function, in out_nX_corr_XXX

clear all; close all;

output=load('out_n2_corr_0.mat');

n = output.n;
sigma = output.sigma;
bar_sigma = output.sigmabar;
beta = output.beta;
D = output.D;
ybar_zzero = output.ybar_zzero;

Nsim = 5000; % number of simulations

for sim=1: Nsim
   
   % start at origin
   p = zeros(n,1);
   time = 0;
   
   y = sum(p.^2);
   z = sum(p);
   
   while y < fun_ybar_adjusted(z,beta,n,ybar_zzero); 
       
       common = randn(1);
       p = p + sqrt(D)*sigma*randn(n,1) + sqrt(D)*bar_sigma*common;
       
       y = sum(p.^2);
       z = sum(p);
       
       time = time + D;
   
   end
   
   psim(1:n,sim) = p ;
   tsim(sim) = time;
   
end

pvec = vec(psim);

zvec = sum([psim  -psim]); % symmetrized distribution of sum of price changes 

Std_Dp = std([pvec ; -pvec]);

phist = [pvec/Std_Dp ; -pvec/Std_Dp]; % standarized (marginal) distribution of price changes

corr = bar_sigma^2/(sigma^2+bar_sigma^2);

% compute correlation between price changes
corr_Dp = cov(psim')/(mean(diag(cov(psim'))));
avg_corr_Dp = mean(corr_Dp(1,2:end));

disp([' correlation of shocks is ', num2str(corr), ' correlation of Dp is ',num2str(avg_corr_Dp) ]);

avg_time=mean(tsim);
avg_Na=1/avg_time; % average number of adjustment per year.


figure(1); hist(phist,50);
ti=title({'Marginal Distribution of $\Delta p$: price changes',...
         ['$n =$ ', num2str(n,2), ', correlation($dp_i,dp_j$) = ',num2str(corr,3)]});
set(ti,'Interpreter','Latex','Fontsize',16);
xl=xlabel('Standardized $\Delta p$');
set(xl,'Interpreter','Latex','Fontsize',16);

minp=min(phist);
maxp=max(phist);


[density,xmesh]= ksdensity(phist,'width',0.07 );

figure(2); plot(xmesh,density);
ti=title({'Marginal Distribution of $\Delta p$: price changes',...
         ['$n =$ ', num2str(n,2), ', correlation($dp_i,dp_j$) = ',num2str(corr,3)]});
set(ti,'Interpreter','Latex','Fontsize',16);
xl=xlabel('Standardized $\Delta p$');
set(xl,'Interpreter','Latex','Fontsize',16);


figure(3); hist(zvec,50);
ti=title({'Marginal Distribution of $z$: sum of price changes',...
             ['$n$ = ', num2str(n,2), ', correlation($dp_i,dp_j$) = ',num2str(corr,3)]});
set(ti,'Interpreter','Latex','Fontsize',16);
xl=xlabel('Standardized $z$');
set(xl,'Interpreter','Latex','Fontsize',16);

% fname = ['simulp_n', num2str(n,2),'_corr_', num2str(corr,3) ,'.mat'];
% 
% sim_output.n       = n;
% sim_output.sigma   = sigma;
% sim_output.sigmabar= bar_sigma;
% sim_output.D       = D;
% sim_output.Nsim    = Nsim ;
% sim_output.pvec    = pvec ;
% sim_output.phist   = phist;
% sim_output.avgcorrDp  = avg_corr_Dp;
% sim_output.avg_Na  = avg_Na;
% sim_output.StdDp   = Std_Dp;
% sim_output.zvec    =  zvec ;
% sim_output.xmesh   =  xmesh;
% sim_output.density =  density ;
%  
% save(fname, '-struct', 'sim_output');
% 


