% This files simulates the process for y and z of firms in the multiproduct
% model with correlation
%
% it takes as inputs:
% n : number of products
% innovation volatitlies sigma and bar_sigam
% boundary of range of innaction, which can be comptudes using the
% paramgers of the value function (a polinomial on y and z), fit.beta
%
% we read all these parameters from a structured created when we solved
% numerically for the value function, in out_nX_corr_XXX

clear all; close all;

output=load('out_n2_corr_0.mat');

n = output.n;
sigma = output.sigma;
bar_sigma = output.sigmabar;
beta = output.beta;
D = output.D;
ybar_zzero = output.ybar_zzero;

Nsim = 10000; % number of simulations

zsim=zeros(1,Nsim);
ysim=zeros(1,Nsim);
tsim=zeros(1,Nsim);

for sim=1: Nsim
   
   % start at origin
   
   time = 0;
   
   y = 0 ;
   z = 0 ;
   
   while y <= fun_ybar_adjusted(z,beta,n,ybar_zzero) ; 
       
  
       e=rand(3,1);
       
       if e(1) <= 1/2
          e_a = -1;
       else
          e_a = 1;
       end
       if e(2) <= 1/2
          e_b = -1;
       else
          e_b = 1;
       end
       if e(3) <= 1/2
          e_c = -1;
       else
          e_c = 1;
       end
            
       [ y z] = fun_law_motion_state(y,z,e_a,e_b,e_c,D,sigma,bar_sigma,n);
       
       
       time = time + D;
   
   end
   
   zsim(sim) = z;
   ysim(sim) = y;
   tsim(sim) = time;
   
end


zvec = [zsim/std(zsim)  -zsim/std(zsim)]; % standardized distribution of sum of price changes 

corr = bar_sigma^2/(sigma^2+bar_sigma^2);

figure(1); hist(ysim,50);
ti=title({'Distribution of $y$ at time of adjustment ',...
         ['$n =$ ', num2str(n,2), ', correlation($dp_i,dp_j$) = ',num2str(corr,3)]});
set(ti,'Interpreter','Latex','Fontsize',16);
xl=xlabel('$y(\tau)$');
set(xl,'Interpreter','Latex','Fontsize',16);

minz=min(zvec);
maxz=max(zvec);

[bandwidth,density,zmesh,cdf]= kde(zvec,2^12,minz,maxz);

figure(2); plot(zmesh,density);
ti=title({'Marginal Distribution of sum of price changes $z$',...
         ['$n =$ ', num2str(n,2), ', correlation($dp_i,dp_j$) = ',num2str(corr,3)]});
set(ti,'Interpreter','Latex','Fontsize',16);
xl=xlabel('Standardized $z$');
set(xl,'Interpreter','Latex','Fontsize',16);


figure(3); hist(zvec,50);
ti=title({'Marginal Distribution of $z$: sum of price changes',...
             ['$n$ = ', num2str(n,2), ', correlation($dp_i,dp_j$) = ',num2str(corr,3)]});
set(ti,'Interpreter','Latex','Fontsize',16);
xl=xlabel('Standardized $z$');
set(xl,'Interpreter','Latex','Fontsize',16);


fname = ['figure_z_n', num2str(n,2),'_corr_', num2str(corr,3) ,'.mat'];

figoutputz.n       = n;
figoutputz.sigma   = sigma;
figoutputz.sigmabar= bar_sigma;
figoutputz.D       = D;
figoutputz.Nsim   =  Nsim ;
figoutputz.zvec   =  zvec ;
figoutputz.ysim   =  ysim ;
figoutputz.xmesh   =  zmesh;
figoutputz.density =  density ;
 
save(fname, '-struct', 'figoutputz');



