% this file computes the frequency of price changes
% and the invariant distribution of y for the multi-product model
% with free adjustment probability lambda

clear all; close all;

% parameters
lambda = 2;
B=20;
sigma = 0.15;
psi_1 = 0.03;
r     = 0.02;

tol = 1e-10;  %  tolerance for quadrature

%n_vec=[1 2 4 10 50];

n_vec=[3 5 10 20  40];

for ii=1:length(n_vec);
    
    n=n_vec(ii);
    psi = psi_1 * n ;
    nu= abs(1-n/2);    
    
    %%%%%%%%%%%%%%%%%%%%%%%
    % compute ybar
    
    % ybar guess with discounting:
    s=sigma^2*(n+2);
    ybar_guess = 2*psi/(-r*psi/s + sqrt( (r*psi/s)^2 + 2*B*psi/s  ) );
    beta0_guess = n*sigma^2/r * 2*psi/ybar_guess;  %  guess value function at y=zero
    
    x0 = [beta0_guess,ybar_guess];
    options = optimset('TolFun',[1e-16],'TolX',[1e-16],'MaxFunEvals',[1000]);
    [Xsol,fval, eflag] = fsolve( @(x) fun_system(x,psi,r,B,sigma,n), x0, options );
    
    beta0  = Xsol(1);
    bar_y  = Xsol(2);
    %%%%%%%%%%%%%%%%%%%%%%%
    
    %   COMPUTE T(0)
    
    % Approx
    N_prod=100;  % #  terms in produttoria
    x = (bar_y*lambda/sigma^2);  
    T_zero  = 1/lambda*fct_lambda_Na(x,n,N_prod) ;   
    Na=1/T_zero;

    % Exact
    if n > 1
        ratio1 = (lambda/(2*sigma^2))^(n/4-1/2) / ...
            ( gamma(n/2)* (bar_y)^(1/2-n/4)*besseli(nu,2*sqrt(lambda*bar_y/(2*sigma^2))) );
        T_zero_anal = (1/lambda)*( 1 - ratio1) ;
    else
        T_zero_anal= T_zero;
    end
    
    const = 1/beta((n-1)/2,1/2);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %  compute kurtosis   
  
    y_dots=2000;
    y_vec=[bar_y/y_dots:bar_y/y_dots:bar_y];
    
    %  compute  - C2/C1  ratio
    ratio = besseli(nu,2*sqrt(bar_y*lambda/(2*sigma^2)))/besselk(nu,2*sqrt(bar_y*lambda/(2*sigma^2)));
    
    CI = 1/(       quad(@(y)     func_besseli_int(y,sigma,lambda,n), y_vec(1), bar_y, tol) ...
          - ratio* quad(@(y) func_besselk_int(y,sigma,lambda,n), y_vec(1), bar_y, tol) ) ;
    CK = -ratio*CI;
       
    %  compute kurtosis by integration of Bessel fcts
    mom_y2 = quad(@(y) y.^2.*CI.*func_besseli_int(y,sigma,lambda,n) + ... 
                       y.^2.*CK.*func_besselk_int(y,sigma,lambda,n), y_vec(1), bar_y, tol);
       
    mom_y1 = quad(@(y) y.*CI.*func_besseli_int(y,sigma,lambda,n) + ... 
                       y.*CK.*func_besselk_int(y,sigma,lambda,n), y_vec(1), bar_y, tol);
    
    kurt_anal(ii)  =   3* n/(2+n) * ( (Na-lambda)/Na * bar_y^2 + lambda/Na* mom_y2 ) ...
                                 ./ ( (Na-lambda)/Na * bar_y   + lambda/Na* mom_y1 ).^2    ; 
                                                       
    % compute kurtosis using discretized grid
    
    for i=1:length(y_vec)
        y        = y_vec(i);
        f_vec(i) = y^(n/4-1/2)*CK*besselk(nu,2*sqrt(y*lambda/(2*sigma^2))) ...
                 + y^(n/4-1/2)*CI*besseli(nu,2*sqrt(y*lambda/(2*sigma^2))) ;
    end
    
    dy = y_vec(3) - y_vec(2);
    
    int_y2=0;int_y1=0;
    for i = 1:length(y_vec)
        int_y2 = int_y2 + y_vec(i)^2*f_vec(i)*dy;
        int_y1 = int_y1 + y_vec(i)*f_vec(i)*dy;
    end
    kurt_vec(ii) = 3*n/(2+n)*(  (Na-lambda)/Na*bar_y^2 + lambda/Na* int_y2  ) / ...
            (  (Na-lambda)/Na*bar_y + lambda/Na* int_y1   )^2;
    
    % Stuff below is old
    %
    dp_vec = [sqrt(bar_y)/400:sqrt(bar_y)/400:sqrt(bar_y)];
    for j=1:length(dp_vec);
        w(j,ii) = 0;
        dp = dp_vec(j);
        if dp/sqrt(bar_y) < 0.99999
            w(j,ii) = (Na-lambda)/Na *const* (1- (dp/sqrt(bar_y))^2)^((n-3)/2) / (sqrt(bar_y));
        end
    end
    
    for k=1:length(y_vec);
        if dp/sqrt(y_vec(k))<0.99999
            w(j,ii) = w(j,ii) + ...
             lambda/Na*const*(1- (dp/sqrt(y_vec(k)))^2)^((n-3)/2) * f_vec(k) * dy / (sqrt(y_vec(k)));
        end
    end
    
    four_mom = 0;
    sec_mom  = 0;
    ddp= dp_vec(3) - dp_vec(2);
    
    for j=1:length(dp_vec)
        four_mon = four_mom + w(j,ii)*dp_vec(j)^4*ddp;
        sec_mon  = sec_mom  + w(j,ii)*dp_vec(j)^2*ddp;
    end
    
    second_mom_vec(ii) = 2*dp_vec.^2*w(:,ii)*ddp;
    fourth_mom_vec(ii) = 2*dp_vec.^4*w(:,ii)*ddp;
    kurt_dp_vec(ii) = fourth_mom_vec(ii)/second_mom_vec(ii)^2 ;
    
    bar_y_vec(ii) = bar_y;
    T_zero_vec(ii) = T_zero;
    T_zero_anal_vec(ii) = T_zero_anal;
    f_mat(:,ii) = f_vec;
    
    T_2nd_order_approx_vec(ii) = bar_y/(n*sigma^2)*(1 - lambda*bar_y/(n*sigma^2)*(n+4)/(2*n+4) ) ;
    %%%%%%%%%%
    %  the stuff above this  line is old
    
end


Na_vec = 1./T_zero_vec;

disp('  ');
disp(' ****** ');

disp(['lambda (number of free arrivals per period)   =  ' , num2str(lambda, 3)]);
disp('  ');
disp(['number of goods  n '  ]);
disp(num2str(n_vec, 3));
disp('  ');
disp(['number of adjustment per period Na'  ]);
disp(num2str(Na_vec, 3));
disp('  ');
disp(['fraction of free adjustments:  lambda / Na'  ]);
disp(num2str(lambda./Na_vec, 3));


disp('  ');
disp('  ');
disp('kurtosis discretized  ');
disp(num2str(kurt_vec, 3));

disp('  ');
disp('kurtosis analytical ');
disp(num2str(kurt_anal, 3));



figure(1);
plot(y_vec,f_mat(:,1)*bar_y_vec(1)/bar_y_vec(5),'b--',y_vec,f_mat(:,2)*bar_y_vec(2)/bar_y_vec(5),'g-',...
    y_vec,f_mat(:,3)*bar_y_vec(3)/bar_y_vec(5),'m-.',y_vec,f_mat(:,4)*bar_y_vec(4)/bar_y_vec(5),'r--',...
    y_vec,f_mat(:,5)*bar_y_vec(5)/bar_y_vec(5),'k.','LineWidth',3);
%axis([0 1 0 1000]);
%axis([0 bar_y*1.05 0 max(max(f_mat))*1.05]);
%axis([0 bar_y 0 30]);
xl=xlabel('$y = ||p||^2$ ','Interpreter','LaTex');set(xl,'Fontsize',18);
yl=ylabel('density of invariant $f(y)$','Interpreter','LaTex');set(yl,'Fontsize',18);
title({'Density $f(y)$ of invariant distribution for different $n$',['$\lambda = $', num2str(lambda,2), ' per year']},...
    'Interpreter','LaTex','Fontsize',18)
le=legend(['n = ',num2str(n_vec(1))] ,['n = ',num2str(n_vec(2))] ,['n = ',num2str(n_vec(3))],...
    ['n = ',num2str(n_vec(4))], ['n = ',num2str(n_vec(5))]); set(le,'Fontsize',18);
te= text(0.99,-65,'$\bar{y}$','Interpreter','LaTex'); set(te,'Fontsize',18);

orient landscape
print -depsc  fig_densities_f_of_y_lambda

% figure(2);
% plot(y_vec,log(f_mat(:,1)*bar_y_vec(1)/bar_y_vec(5)),'b--',y_vec,log(f_mat(:,2)*bar_y_vec(2)/bar_y_vec(5)),'g-',...
%     y_vec,log(f_mat(:,3)*bar_y_vec(3)/bar_y_vec(5)),'m-.',y_vec,log(f_mat(:,4)*bar_y_vec(4)/bar_y_vec(5)),'r--',...
%     y_vec,log(f_mat(:,5)*bar_y_vec(5)/bar_y_vec(5)),'k.','LineWidth',3);
% grid on
% axis([0 1 0 1000]);
% axis([0 bar_y*1.05 0 max(max(f_mat))*1.05]);
% axis([0 bar_y 0 30]);
% axis([0 max(dp_vec) log(f_mat(end-3,5)) max(max(log(f_mat)))])
% xl=xlabel('$y = ||p||^2$ ','Interpreter','LaTex');set(xl,'Fontsize',18);
% yl=ylabel('log of density of invariant $f(y)$','Interpreter','LaTex');set(yl,'Fontsize',18);
% title({'(log of ) Density $f(y)$ of invariant distribution for different $n$',['$\lambda = $', num2str(lambda,2), ' per year']},...
%     'Interpreter','LaTex','Fontsize',18)
% le=legend(['n = ',num2str(n_vec(1))] ,['n = ',num2str(n_vec(2))] ,['n = ',num2str(n_vec(3))],...
%     ['n = ',num2str(n_vec(4))], ['n = ',num2str(n_vec(5))]); set(le,'Fontsize',18);
% te= text(0.99,-65,'$\bar{y}$','Interpreter','LaTex'); set(te,'Fontsize',18);

std = sqrt(bar_y_vec(end)/n_vec(end));
density_normal= normpdf((1/std)*dp_vec,0,1);

figure(3);
plot(dp_vec*sqrt(bar_y_vec(1)/bar_y_vec(5)),w(:,1),'b--',dp_vec*sqrt(bar_y_vec(2)/bar_y_vec(5)),w(:,2),'g-',...
    dp_vec*sqrt(bar_y_vec(3)/bar_y_vec(5)),w(:,3),'m-.',dp_vec*sqrt(bar_y_vec(4)/bar_y_vec(5)),w(:,4),'r--',...
    dp_vec,w(:,5),'k.',dp_vec,density_normal,'LineWidth',2);
%axis([0 .2 0 20]);
%axis([0 bar_y*1.05 0 max(max(f_mat))*1.05]);
%axis([0 bar_y 0 30]);
xl=xlabel('$\Delta p$ ','Interpreter','LaTex');set(xl,'Fontsize',18);
yl=ylabel('density $w(\Delta p)$','Interpreter','LaTex');set(yl,'Fontsize',18);
title({'Density $w(\Delta p)$ for different $n$',['$\lambda = $', num2str(lambda,2), ' per year']},...
    'Interpreter','LaTex','Fontsize',18)
le=legend(['n = ',num2str(n_vec(1))] ,['n = ',num2str(n_vec(2))] ,['n = ',num2str(n_vec(3))],...
    ['n = ',num2str(n_vec(4))], ['n = ',num2str(n_vec(5))], 'normal'); set(le,'Fontsize',18);
te= text(0.99,-65,'$\bar{y}$','Interpreter','LaTex'); set(te,'Fontsize',18);



