%  Code for cumulative IRF after a monetary shock
%  obtained by simulation, loops over delta for a fixed n 
%
clear all; close all;

%  number of goods
n = 10 ;

%  whether (==1)  or not (==0) you want figures
figures  = 1   ; 
%  whether (==1)  or not (==0) save output 
saveoutput=1   ;

% Data inputs from steady state observations

std_Dp = .1 ; % std dev price changes
Na     =  1 ; % expected number of price changes

% vector of values of delta
 delta_vec = [std_Dp/10:std_Dp/10:std_Dp] ; % vector of n to compute IRFs
% delta_vec = [0.01] % [.01  .05  .1]

tmax  =  5 ;  %  max number of years to keep track of firms

dt      =  1/365/2 ;  %1/24     ;  %  period length (1/12 is a month)
n_firms =  10*100000 ;            % number of firms to simulate

% vector of percentiles , from inverse of F(y) used to initialize price gaps
F_vec = linspace(0,1,n_firms/2);

% implied values of ybar and sigma
ybar  =   std_Dp^2  * n    ;    % implied ybar to match  sdt_Dp
sigma =   sqrt(ybar*Na/n)  ; % annual volatility of shocks to match Na

sqrt_ybar = sqrt(ybar);


% first we assign to each firm a value of ybar with the invariant density f(y)

y_vec(1) = 0;
y_vec(2:n_firms/2-1) = fct_inv_F(F_vec(2:n_firms/2-1),n,ybar);
y_vec(n_firms/2) = ybar;

tic;

%  Create a sample from the invariant distribution
%  first do it for half of the firms then flip it
for i = 1 : n_firms/2
    
    y =  y_vec(i);
    
    XX = randn(n,1) ;
    
    P(:,i) = sqrt(y)*XX/norm(XX);
    
    Psim(:,i)=  -P(:,i);
    
end

toc;

PP0 =[P,Psim];


for id=1:length(delta_vec);
    
    delta     = delta_vec(id)     ;  % value of delta
    
    PP = PP0 - delta*ones(size(PP0));  %  vector of prices after monetary shock
    
    
    % next loop take each of the firm's p -a draw from the invariant,
    % and simulate their price gaps until adjustment.
    % once they adjust we keep the price adjustment and the tiem of the
    % adjusment.
    % Note that each firm is replaced by a new one, with p=0, that contribute
    % zero to the price adjustment.
    
    for i=1:n_firms
        
        t=1;
        Pi = PP(:,i)  ;
        
        % t=1 correspond to the jump
        
        if  norm(Pi)  >= sqrt_ybar
            Dp(i)      =  - mean(Pi) ; %  average price change for the n firm products
            ta(i)      = t ;          %  keep how long it took to adjust
            Theta_i(i) = Dp(i);
            
        else %there is no jump at time zero (t=1)
            t = t+1;
            % immitate the continous time n dimensional BMs
            Pi = Pi + sigma*sqrt(dt)*randn(n,1);
                        
            while  norm(Pi) < sqrt_ybar  &&  t < tmax/dt
                t = t+1;
                % immitate the continous time n dimensional BMs
                Pi = Pi + sigma*sqrt(dt)*randn(n,1);
            end
            
            Dp(i) =  - mean(Pi) ; %  average price change for the n firm products
            ta(i) = t ;          %  keep how long it took to adjust   
            Theta_i(i)=0 ; % there was no jump at time zero
            
        end  % at the end of this if either jump at t=1 or adjust later 
        
        
        % continue if price gaps^2 < ybar
       
    end
    
    % Construct the IR function
    
    % sort the firms (and their price changes) using the time they adjusted prices:
    
    [ta_sorted,indx_ta]=sort(ta);
    
    Dp_sort = Dp(indx_ta)  ;
    
    Theta(id)   = mean(Theta_i);
    % initialize cumulative change in aggregate prices
    Cum_Y(id) = 0;
    
    % Loops over all the times (horizons) for the IRF function
    % for each horizon compute: fraction that adjust (frac_adj) and
    % contribution to inflation (D_CPI):
    
  %    compute time zero irf  t=1  is time zero 
        
    j = 1 ;  %  i.e. if there was a jump at time zero
    indx_adj     = find(ta_sorted==j); % find all the firms that adjust in period j
    frac_adj(j)  = length(indx_adj)/n_firms;
    D_CPI(j)     = sum(Dp_sort(indx_adj))/n_firms;
    Cum_CPI(j)   = D_CPI(j) ;
    Y            = delta - Cum_CPI(j) ;
    Cum_Y(id)    = Y*dt  + Cum_Y(id)  ;
        
% compute irf for periods after shock        
        
    for j = 2 : max(ta_sorted)
    
        indx_adj = find(ta_sorted==j); % find all the firms that adjust in period j
        
        if isempty(indx_adj) % case at which no firm adjusted at horizon j
            frac_adj(j)=0;
            D_CPI(j)=0;
            Cum_CPI(j) = Cum_CPI(j-1) + D_CPI(j) ;
        else
            frac_adj(j)  = length(indx_adj)/n_firms;
            D_CPI(j)     = sum(Dp_sort(indx_adj))/n_firms;
            Cum_CPI(j)   = Cum_CPI(j-1) + D_CPI(j) ;
            Y            = delta - Cum_CPI(j)      ;
            Cum_Y(id)    = Y*dt + Cum_Y(id)        ; 
        end
        
    end
    
%   Cum_CPI = Cum_CPI(1:end) ;

%  stop
        
    % Save structure with inputs and output
    
    if saveoutput==1
    
    fname = ['output_n', num2str(n,2),'_d', num2str(delta,2),'.mat'];
    
    output_d.n       = n;
    output_d.dt      = dt;
    output_d.tmax    = tmax;
    output_d.ybar    = ybar;
    output_d.sigma   = sigma;
    output_d.delta   = delta;
    output_d.nf      = n_firms;
    output_d.frac    = frac_adj;
    output_d.Theta   = Theta(id);
    output_d.Dcpi    = D_CPI;
    output_d.Cum_cpi = Cum_CPI;
    output_d.Cum_Y   = Cum_Y(id);
    
    save(fname, '-struct', 'output_d');
    
    fname_vec{id} = fname  ; 
    
    end
    
    clear PP P Psim Cum_CPI D_CPI frac_adj ta Dp ;
    clear ta_sorted DP_sort indx_adj Y
    
end


% stop 

if saveoutput==1
fname = ['out_n', num2str(n,2),'_MULTI_d','.mat'];

output_d.delta_vec    = delta_vec;
output_d.n       = n;
output_d.dt      = dt;
output_d.tmax    = tmax;
output_d.ybar    = ybar;
output_d.sigma   = sigma;
output_d.nf      = n_firms;
output_d.Cum_Y   = Cum_Y;
output_d.Theta   = Theta;
output_d.names   = fname_vec;
save(fname, '-struct', 'output_d');

end

% load ouput MULTI_d  to see next fig
figure(1)
plot(delta_vec,Cum_Y,'b-');
title(['Cumulative output response for n = ', num2str(n)]); 

if figures==1

entry1=fname_vec{1};
entry2=fname_vec{2};
entry3=fname_vec{3};

%  produce some figures after you have simulation for various n
out_1 = load(entry1);
out_2 = load(entry2);
out_3 = load(entry3);

% t range for plots
irf_periods=2/dt;
t_vec = [1:1:irf_periods];

lab1 =  [' d = ',  num2str(out_1.delta)] ;
lab2 =  [' d = ',  num2str(out_2.delta)] ;
lab3 =  [' d = ',  num2str(out_3.delta)] ;

figure(2)
plot(t_vec,out_1.frac(t_vec),'b-',t_vec, out_2.frac(t_vec),'r--',t_vec, out_3.frac(t_vec),'k-.')
title([' Fraction of adjusters after a monetary shock  of size d for  n = ', num2str(n)])
legend(lab1,lab2,lab3)
 
figure(3)
plot(t_vec,out_1.Cum_cpi(t_vec),'b-',t_vec, out_2.Cum_cpi(t_vec),'r--',t_vec,out_3.Cum_cpi(t_vec),'k-.')
title([' CPI response to a monetary shock  of size d  for n = ', num2str(n)])
legend(lab1,lab2,lab3)

end

