%  Code for IRF after a monetary shock
%  obtained by simulation, loops over n 

clear all; close all;


n_vec =  [1 2 3 30]; % vector of n to compute IRFs


%  whether (==1)  or not (==2) you want figures
figures  = 1   ; 


% data inputs  from steady state observations

std_Dp =   .1   ; % std dev price changes
Na     =    1   ; % expected number of price changes

% params
delta =  0.01 % 0.01/2 % 0.01  ;  % size of the monetary shock

tmax  =  5 ;  %  max number of years to keep track of firms

dt      =  1/365/2 %1/24     ;  %  period length (1/12 is a month)
n_firms =  500000 ;      % number of firms to simulate

% vector of percentiles , from inverse of F(y) used to initialize price gaps
F_vec = linspace(0,1,n_firms/2);

for in=1:length(n_vec);
    
    n     = n_vec(in)     ;  % number of products
    
    
    % implied values of ybar and sigma
    ybar  =   std_Dp^2  * n ;    % implied ybar to match  sdt_Dp
    sigma =   sqrt(ybar*Na/n)  ; % annual volatility of shocks to match Na
    
    % first we assign to each firm a value of ybar with the invariant density f(y)
    
    y_vec(1) = 0;
    y_vec(2:n_firms/2-1) = fct_inv_F(F_vec(2:n_firms/2-1),n,ybar);
    y_vec(n_firms/2) = ybar;
    
    tic;
    
    %  Create a sample from the invariant distribution
    %  first do it for half of the firms then flip it
    for i = 1 : n_firms/2
        
        y =  y_vec(i);
        
        XX = randn(n,1) ;
        
        P(:,i) = sqrt(y)*XX/norm(XX);
        
        Psim(:,i)=  -P(:,i);
        
    end
    
    toc;
    
    PP =[P,Psim];
    
    %delta=.5
    PP = PP -delta*ones(size(PP));  %  vector of prices after monetary shock
    
    sqrt_ybar=sqrt(ybar);
    
    % next loop take each of the firm's p -a draw from the invariant,
    % and simulate their price gaps until adjustment.
    % once they adjust we keep the price adjustment and the tiem of the
    % adjusment.
    % Note that each firm is replaced by a new one, with p=0, that contribute
    % zero to the price adjustment.
    
    for i=1:n_firms
        
        t=1;
        Pi = PP(:,i)  ;
        
        % continue if price gaps^2 < ybar
        while  norm(Pi)<sqrt_ybar  &&  t < tmax/dt
            t = t+1;
            % imitate the continous time n dimensional BMs
            Pi = Pi + sigma*sqrt(dt)*randn(n,1);
        end
        
        Dp(i) =  - mean(Pi); % average price change for the n firm products
        ta(i) = t ;          %  keep how long it took to adjust
        
    end
    
    
    % Construct the IR function
    
    
    % sort the firms (and their price changes) in the usign the time they
    % adjusted prices:
    
    [ta_sorted,indx_ta]=sort(ta);
    
    Dp_sort = Dp([indx_ta])  ;
    
    % initialize cumulative change in aggregate prices
    Cum_CPI(1)=0;
    
    % loops over all the times (horizons) for the IRF function
    % for each horizon compute: fraction that adjust (frac_adj) and
    % contritubiton to inflation (D_CPI):
    
    for j = 1 : max(ta_sorted)
        
        indx_adj = find(ta_sorted==j); % find all the firms that adjust in period j
        
        if isempty(indx_adj) % case at which no firm adjusted at horizon j
            frac_adj(j)   = 0;
            D_CPI(j)      = 0;
            Cum_CPI(j+1)  = Cum_CPI(j) + D_CPI(j) ;
            
        else
            frac_adj(j)  = length(indx_adj)/n_firms;
            D_CPI(j)     = sum(Dp_sort(indx_adj))/n_firms;
            Cum_CPI(j+1) = Cum_CPI(j) + D_CPI(j) ;
        end
        
    end
    
    Cum_CPI = Cum_CPI(2:end) ;
        
    % save structure with inputs and output
    
    fname = ['output_n', num2str(n,2) ,'.mat'];
    
    output_n.n       = n;
    output_n.ybar    = ybar;
    output_n.sigma   = sigma;
    output_n.delta   = delta;
    output_n.nf      = n_firms;
    output_n.frac    = frac_adj;
    output_n.Dcpi    = D_CPI;
    output_n.Cum_cpi = Cum_CPI;
    save(fname, '-struct', 'output_n');
    
    fname_vec{in} = fname  ; 
    
    clear PP P Psim Cum_CPI D_CPI frac_adj ybar sigma y_vec sqrt_ybar ta DP ;
    clear ta_sorted DP_sort indx_adj
    
end

if figures==1
    
    
    if length(n_vec)==2;
        
        entry1=fname_vec{1};
        entry2=fname_vec{2};
        
        %  produce some figures after you have simulation for various n
        out_1 = load(entry1);
        out_2 = load(entry2);
        % out_3 = load(entry3);
        
        % t range for plots
        irf_periods=50;
        t_vec = [1:1:irf_periods];
        
        lab1 =  [' n = ',  num2str(out_1.n)] ;
        lab2 =  [' n = ',  num2str(out_2.n)] ;
        
        figure(1)
        plot(t_vec,out_1.frac(t_vec),'b-',t_vec, out_2.frac(t_vec),'r--')
        title([' Fraction of adjusters after a ',num2str(delta*100,2),' percent monetary shock'])
        legend(lab1,lab2)
        
        
        figure(2)
        plot(t_vec,out_1.Cum_cpi(t_vec),'b-',t_vec, out_2.Cum_cpi(t_vec),'r--')
        title([' CPI response to a ',num2str(delta*100,2),' percent monetary shock'])
        legend(lab1,lab2)
        
    else  % more than 2 entries  for n
        
        entry1=fname_vec{1};
        entry2=fname_vec{2};
        entry3=fname_vec{3};
        
        %  produce some figures after you have simulation for various n
        out_1 = load(entry1);
        out_2 = load(entry2);
        out_3 = load(entry3);
        
        % t range for plots
        irf_periods=50;
        t_vec = [1:1:irf_periods];
        
        lab1 =  [' n = ',  num2str(out_1.n)] ;
        lab2 =  [' n = ',  num2str(out_2.n)] ;
        lab3 =  [' n = ',  num2str(out_3.n)] ;
        
        figure(1)
        plot(t_vec,out_1.frac(t_vec),'b-',t_vec, out_2.frac(t_vec),'r--',t_vec, out_3.frac(t_vec),'k-.')
        title([' Fraction of adjusters after a ',num2str(delta*100,2),' percent monetary shock'])
        legend(lab1,lab2,lab3)
        
        figure(2)
        plot(t_vec,out_1.Cum_cpi(t_vec)*100,'b-',t_vec, out_2.Cum_cpi(t_vec)*100,'r--',t_vec,out_3.Cum_cpi(t_vec)*100,'k-.')
        title([' CPI response to a ',num2str(delta*100,2),' percent monetary shock'])
        legend(lab1,lab2,lab3)
        
    end
    
end