% this script computes the cumulative value of impulse responses 
% for output for n=1 and n=\infty;  it also draws a figure with
% n=1,2,10,infty

% close all
clear all; 

%xvec=[-1:0.001:1]; 
Na    =   1 ;%.24*12;
pbar  =  .1;
st_Dp =   pbar;

sigma = sqrt(Na) * pbar  ;  
k_num = 20;

delta_vec=[0:pbar/50:pbar];

for i=1:length(delta_vec);
    
    delta = delta_vec(i);
    
    % computes the cumulative outpout IRF for n=1;
    YCum_1(i)=fct_YI_anal(delta,pbar,sigma,k_num);
    
    % computes the cumulative outpout IRF for n=infty;
    if delta <= st_Dp;
        Phi = (delta/st_Dp)^2;
        T = (1-Phi)/Na;
        YCum_infty(i) = delta*(1-Phi)*T - delta*Na*T^2/2;
    else
        YCum_infty(i) = 0;
    end
end

[max_1,d1]=max(YCum_1);
[max_infty,di]=max(YCum_infty);

figure(10);
pl=plot(100*delta_vec,100*YCum_1,'k-',100*delta_vec,100*YCum_infty,'r--');
set(pl,'LineWidth',2)
xl=xlabel('monetary shock $\delta$ in percentage');
set(xl,'Interpreter','Latex','Fontsize',18);
%yl=ylabel('Cumulative ``Output" effect $\int_0^{\infty}\, \left(\delta - P_n(\delta,t)\right)\ \ dt$, in \%');
yl=ylabel('Cumulative ``Output" effect $\mathcal{M}(\delta)$, in \%');
set(yl,'Interpreter','Latex','Fontsize',18);
le=legend('$n = 1$','$n = \infty$', 'Location','Best');
set(le,'Interpreter','Latex','Fontsize',18);
%ti=title(['Cumulative ``Output" deviation, $Std(\Delta p) = $',num2str(pbar,2) ,' and  $ N_a = $', num2str(Na)]);
%set(ti,'Interpreter','Latex','Fontsize',18);
vline(100*delta_vec(d1));
vline(100*delta_vec(di));
hline(100*max_1);
hline(100*max_infty);

% orient landscape
% print -depsc  fig_cumY_IRF_1_infty


%  now load data for n=2 

out2 =load('out_n2_MULTI_d.mat');
pol_degree=4;
Pol2 = polyfit([0 out2.delta_vec], [0 out2.Cum_Y], pol_degree);

out10 =load('out_n10_MULTI_d.mat');
pol_degree=4;
Pol10 = polyfit([0 out10.delta_vec], [0 out10.Cum_Y], pol_degree);


figure(20);
pl=plot(100*delta_vec,100*YCum_1,'m-',100*delta_vec,100*polyval(Pol2,delta_vec),'r-.',100*delta_vec,100*polyval(Pol10,delta_vec),'b:',100*delta_vec,100*YCum_infty,'k--' );
% plot(100*delta_vec,100*polyval(Pol2,delta_vec),'m-.' );


set(pl,'LineWidth',2)
xl=xlabel('monetary shock $\delta$ in percentage');
set(xl,'Interpreter','Latex','Fontsize',18);
%yl=ylabel('Cumulative ``Output" effect $\int_0^{\infty}\, \left(\delta - P_n(\delta,t)\right)\ \ dt$, in \%');
yl=ylabel('Cumulative ``Output" effect $\mathcal{M}(\delta)$, in \%');
set(yl,'Interpreter','Latex','Fontsize',18);
le=legend('$n = 1$','$n = 2$','$n = 10$', '$n = \infty$', 'Location','Best');
set(le,'Interpreter','Latex','Fontsize',18);
%ti=title(['Cumulative ``Output" deviation, $Std(\Delta p) = $',num2str(pbar,2) ,' and  $ N_a = $', num2str(Na)]);
%set(ti,'Interpreter','Latex','Fontsize',18);
vline(100*delta_vec(d1));
vline(100*delta_vec(di));
hline(100*max_1);
hline(100*max_infty);

orient landscape
print -depsc  fig_cumY_final
