%  Code for IRF after a monetary shock
%  obtained by simulation
%  you should run solve_irf.m  first or use its saved output

clear all; close all;

% select what n  you want to display 

n_vec=[1 2 3 10 30]; % vector of n to compute IRFs

irf_periods=60;
dt   =  1/52     ;  %  period length (1/12 is a month, 1/52 is weekly)

out_1 = load(['output_n',num2str(n_vec(1)),'.mat']);
out_2 = load(['output_n',num2str(n_vec(2)),'.mat'] );
out_3 = load(['output_n',num2str(n_vec(3)),'.mat']);
out_4 = load(['output_n',num2str(n_vec(4)),'.mat']);
out_5 = load(['output_n',num2str(n_vec(5)),'.mat'] );
 
delta = out_1.delta*100;

% t range for plots
t_vec = [1:1:irf_periods];

lab1 =  [' n = ',  num2str(out_1.n)] ;
lab2 =  [' n = ',  num2str(out_2.n)] ;
lab3 =  [' n = ',  num2str(out_3.n)] ;
lab4 =  [' n = ',  num2str(out_4.n)] ;
lab5 =  [' n = ',  num2str(out_5.n)] ;


figure(1)
plot(t_vec,out_1.frac(t_vec),'b-',t_vec, out_2.frac(t_vec),'r--',...
     t_vec, out_3.frac(t_vec),'k-.',t_vec, out_4.frac(t_vec),'g-.',t_vec, out_5.frac(t_vec),'c-.')
title([' Fraction of adjusters after a ',num2str(delta,2),' percent monetary shock'])
legend(lab1,lab2,lab3,lab4,lab5)
 
figure(2)
plot(t_vec,out_1.Cum_cpi(t_vec),'b-',t_vec, out_2.Cum_cpi(t_vec),'r--',...
    t_vec,out_3.Cum_cpi(t_vec),'k-.',t_vec, out_4.Cum_cpi(t_vec),'g-.',t_vec,out_5.Cum_cpi(t_vec),'c-.')
title([' CPI response to a ',num2str(delta,2),' percent monetary shock'])
legend(lab1,lab2,lab3,lab4,lab5)



%  average variables over months 

% aggregate ar monthly  frequency

t_month = [1:20];

for i = 1:length(t_month)
    
frac1_month(i)=2*mean( out_1.frac(i*2-1:i*2) ) ;
frac2_month(i)=2*mean( out_2.frac(i*2-1:i*2) ) ;
frac3_month(i)=2*mean( out_3.frac(i*2-1:i*2) ) ;
frac4_month(i)=2*mean( out_4.frac(i*2-1:i*2) ) ;
frac5_month(i)=2*mean( out_5.frac(i*2-1:i*2) ) ;


Dcpi1_month(i)=100*sum( out_1.Dcpi(i*2-1:i*2) ) ;
Dcpi2_month(i)=100*sum( out_2.Dcpi(i*2-1:i*2) ) ;
Dcpi3_month(i)=100*sum( out_3.Dcpi(i*2-1:i*2) ) ;
Dcpi4_month(i)=100*sum( out_4.Dcpi(i*2-1:i*2) ) ;
Dcpi5_month(i)=100*sum( out_5.Dcpi(i*2-1:i*2) ) ;

cpi1_month(i)=100*mean( out_1.Cum_cpi(i*2-1:i*2) ) ;
cpi2_month(i)=100*mean( out_2.Cum_cpi(i*2-1:i*2) ) ;
cpi3_month(i)=100*mean( out_3.Cum_cpi(i*2-1:i*2) ) ;
cpi4_month(i)=100*mean( out_4.Cum_cpi(i*2-1:i*2) ) ;
cpi5_month(i)=100*mean( out_5.Cum_cpi(i*2-1:i*2) ) ;

end

figure(3)
ph=plot(t_month,frac1_month,'b-',t_month,frac2_month,'r--',...
     t_month,frac3_month,'k-.',t_month,frac4_month,'g-.',t_month,frac5_month,'c-.');
set(ph,'Linewidth',2) 
title([' Fraction of adjusters after a ',num2str(delta,2),' percent monetary shock'],'Interpreter','Latex','Fontsize',16)
legend(lab1,lab2,lab3,lab4,lab5)
xlabel('months','Interpreter','Latex','Fontsize',14)
ylabel('fraction of firms adjusting','Interpreter','Latex','Fontsize',14)


Na=out_1.n*out_1.sigma^2/out_1.ybar;
StdDp= sqrt(out_1.ybar/out_1.n);

figure(4)
ph=plot(t_month,cpi1_month,'b-',t_month,cpi2_month,'r--',...
     t_month,cpi3_month,'k-.',t_month,cpi4_month,'g-.',t_month,cpi5_month,'c-.');
set(ph,'Linewidth',2)
axis([0 t_month(end) 0 delta*1.1])
title([num2str(delta,2), ' percent monetary shock, $N_a = $ ',...
    num2str(Na,2),', $Std[\Delta p] = $',num2str(StdDp,2)],'Interpreter','Latex','Fontsize',20)
le=legend(lab1,lab2,lab3,lab4,lab5,'Location','SouthEast');
grid on
set(le,'Interpreter','Latex','Fontsize',16)
hline(delta*1);
hline(delta*0.5);
xlabel('months','Interpreter','Latex','Fontsize',18)
ylabel('CPI response, in percentage ','Interpreter','Latex','Fontsize',18)

text(t_month(end)*0.7,0.7*delta,['$\frac{\delta}{Std[\Delta p]} \, = $ ',num2str(delta/100/StdDp,1)],'Interpreter','Latex','Fontsize',18)


% keep the statistics at dt period

cpi1_dt=out_1.Cum_cpi;
cpi2_dt=out_2.Cum_cpi;
cpi3_dt=out_3.Cum_cpi;
cpi4_dt=out_4.Cum_cpi;
cpi5_dt=out_5.Cum_cpi;

T_2w = length(cpi1_dt);
if length(cpi2_dt) < T_2w
   for i=length(cpi2_dt)+1: T_2w
   cpi2_dt(i)= delta/100;
   end
end

if length(cpi3_dt) < T_2w
   for i=length(cpi3_dt)+1: T_2w
   cpi3_dt(i)= delta/100;
   end
end

if length(cpi4_dt) < T_2w
   for i=length(cpi4_dt)+1: T_2w
   cpi4_dt(i)= delta/100;
   end
end

if length(cpi5_dt) < T_2w
   for i=length(cpi5_dt)+1: T_2w
   cpi5_dt(i)= delta/100;
   end
end

dt_p_m=1/12/dt; % dt_p_m : dt per month,
Tmax=round(max(t_month)*dt_p_m);
figure(41)
t_dt=[1:1:length(cpi1_dt)];
ph=plot(t_dt(1:Tmax)/dt_p_m,cpi1_dt(1:Tmax)*100,'b-',t_dt(1:Tmax)/dt_p_m,cpi2_dt(1:Tmax)*100,'r--',...
     t_dt(1:Tmax)/dt_p_m,cpi3_dt(1:Tmax)*100,'k-.',t_dt(1:Tmax)/dt_p_m,cpi4_dt(1:Tmax)*100,'g-.',t_dt(1:Tmax)/dt_p_m,cpi5_dt(1:Tmax)*100,'c-.');
set(ph,'Linewidth',2)
axis([0 t_dt(Tmax)/dt_p_m 0 delta*1.1])
title([num2str(delta,2), ' percent monetary shock, $N_a = $ ',...
    num2str(Na,2),', $Std[\Delta p] = $',num2str(StdDp,2)],'Interpreter','Latex','Fontsize',20)
le=legend(lab1,lab2,lab3,lab4,lab5,'Location','SouthEast');
grid on
set(le,'Interpreter','Latex','Fontsize',16)
hline(delta*1,'k','');
hline(delta*0.5,'k','');
xlabel('months','Interpreter','Latex','Fontsize',18)
ylabel('CPI response, in percentage ','Interpreter','Latex','Fontsize',18)

text(t_dt(Tmax)/dt_p_m*0.7,0.7*delta,['$\frac{\delta}{Std[\Delta p]} \, = $ ',num2str(delta/100/StdDp,1)],'Interpreter','Latex','Fontsize',18)

orient landscape 
print -depsc  fig_IRF


figure(42)
grid off
ph=plot(t_month,Dcpi1_month,'b-',t_month,Dcpi2_month,'r--',...
     t_month,Dcpi3_month,'k-.',t_month,Dcpi4_month,'g-.',t_month,Dcpi5_month,'c-.');
set(ph,'Linewidth',2) 
title([' D-CPI response after a ',num2str(delta,2),' percent monetary shock'],'Interpreter','Latex','Fontsize',16)
legend(lab1,lab2,lab3,lab4,lab5)
xlabel('months','Interpreter','Latex','Fontsize',14)
ylabel('D CPI  ','Interpreter','Latex','Fontsize',14)


figure(5)
ph=plot(t_month,delta-cpi1_month,'b-',t_month,delta-cpi2_month,'b.',...
     t_month,delta-cpi3_month,'b-.',t_month,delta-cpi4_month,'b--',t_month,delta-cpi5_month,'b:');
set(ph,'Linewidth',2) 
axis([0  18  -.1  1])
title([' ``Output"   response after a ',num2str(delta,2),' percent monetary shock'],'Interpreter','Latex','Fontsize',16)
legend(lab1,lab2,lab3,lab4,lab5)
xlabel('months','Interpreter','Latex','Fontsize',14)
ylabel(' ``Output"  $\delta - \Theta_{n,t} $','Interpreter','Latex','Fontsize',16)
hline(0)



figure(6)
ph=plot(t_month,cpi2_month./cpi1_month,'r--',t_month,cpi3_month./cpi1_month,'k-.',...
        t_month,cpi4_month./cpi1_month,'g-.',t_month,cpi5_month./cpi1_month,'c-.');
set(ph,'Linewidth',2) 
title([' CPI  relative to n=1 case ,   after ',num2str(delta,2),' percent monetary shock'],'Interpreter','Latex','Fontsize',16)
legend(lab2,lab3,lab4,lab5)
xlabel('months','Interpreter','Latex','Fontsize',16)
ylabel('$\Theta_{n,t} \ \  / \  \ \Theta_{1,t}$','Interpreter','Latex','Fontsize',18)

