% this files computes the hazard rate for the multi-product model
%  
% R = sum n indepdenden BM squares, with std sigma
% reflecting on the boundary barR to the origin
%
% dR = n sigma^2 dt + 2 sigma sqrt(R) dW in R in [0 Rbar]
%
% R is returned to zero when reaches Rbar.
%
% we use the formulae in the paper.


n_vec = [1, 2, 4, 10, 50];

t_vec = [1/50:1/50:3];

avg_time = 1;
sigma=0.2;
Nr=300; % number of terms in sums

vec_q=[''];
vec_qsq=[''];

for nn=1:length(n_vec)
    
    if n_vec(nn) == 1
        n=1;
        
        Rbar = avg_time*n*sigma^2;
        for r=1:Nr;
            j=r-1;
            xi(r) = (2*j+1)*((-1)^j)*pi*sigma^2/(2*Rbar);
            e(r)=(2*j+1)^2*pi^2*sigma^2/(8*Rbar);
        end
        
        % S is the survival function, h the hazard
        for i=1:length(t_vec);
            t=t_vec(i);
            S(i,nn) = 0;
            h(i,nn) = 0;
            for r=1:Nr
                S(i,nn) = S(i,nn) + xi(r)/e(r)*exp(-e(r)*t);
                
                h(i,nn) = h(i,nn) + xi(r)*exp(-e(r)*t);
            end
            h(i,nn) = h(i,nn)/S(i,nn);
        end
        
        avg_t(nn) = Rbar/(n*sigma^2);
        
    else
        
        n=n_vec(nn);
        
        Rbar = avg_time*n*sigma^2; % chose Rbar so that avg_time til hit Rbar =1
        
        nu = n/2 -1; % parameter for Bessel function
        
        clear q ;
        % find the zeroes of the Bessel function
        qq=besselzero(nu,Nr,1);
        if n==2
            q = real(qq);
        else
            q=qq;
        end
        
        clear xi;
        
        % compute coefficients xi
        for r=1:Nr;
            xi(r) = q(r)^(nu-1)/( 2^(nu-1)*gamma(nu+1)*besselj(nu+1,q(r)) );
        end
        
        % S is the survival function, h is the hazard
        for i=1:length(t_vec);
            t=t_vec(i);
            S(i,nn) = 0;
            h(i,nn) = 0;
            for r=1:Nr
                S(i,nn) = S(i,nn) + xi(r)*exp(-q(r)^2*sigma^2/(2*Rbar)*t);
                h(i,nn) = h(i,nn) + xi(r)*q(r)^2*sigma^2/(2*Rbar)*exp(-q(r)^2*sigma^2/(2*Rbar)*t);
            end
            h(i,nn) = h(i,nn)/S(i,nn);
        end
        
        avg_t(nn) = Rbar/(n*sigma^2);
        
        vec_q = [vec_q, num2str(q(1),3),' & '];
        vec_qsq = [vec_qsq, num2str(q(1)^2/(n*2),3),' & '];
        
    end
    
end


figure(1);
plot(t_vec,h(:,1),'b--',t_vec,h(:,2),'g-',t_vec,h(:,3),'m-.',t_vec,h(:,4),'r--',...
    t_vec,h(:,5),'k.','LineWidth',3); 
axis([0 max(t_vec) -0.001 max(h(end,:))*1.01]);
xl=xlabel('Time elapsed since last adjustment','Interpreter','LaTex');set(xl,'Fontsize',14);
yl=ylabel('Instantaneous hazard rate','Interpreter','LaTex');set(yl,'Fontsize',14);
%title({'Hazard rate of price adjusment for different n','n: number of products'},'Interpreter','LaTex')
le=legend('n = 1' ,'n = 2' ,'n = 4' ,'n = 10', 'n = 50','Location','East');
set(le,'Fontsize',14);
te= text(0.8,0.2,'Expected Duration','Interpreter','LaTex'); set(te,'Fontsize',12);
vl=vline(1,'b-','');  %set(vl,'Fontsize',14);

print -depsc  fig_hazard_multi


nn_vec=[1,2,3,4,6,8,10,20,50,100];

clear vec_qq; clear vec_qqsq; clear nn_qq;
vec_qq=[''];
vec_qqsq=[''];
nn_qq=[''];

for nnn=1:length(nn_vec);
        n = nn_vec(nnn);
        if nn_vec(nnn) ==1;
        
        nn_qq = [nn_qq, '  &  ',num2str(n,1)];    
        vec_qq   = [vec_qq, ' & ', 'n.a.'];
        vec_qqsq = [vec_qqsq, ' & ', num2str(pi^2/8,2)];
           
        else
            
        nu = n/2-1;
        if n == 2; 
            qqzero= besselzero(nu,1,1);
            qzero=real(qqzero);
        else
            qzero = besselzero(nu,1,1);
        end
        nn_qq = [nn_qq, '  &  ',num2str(n,3)];
        vec_qq   = [vec_qq, ' & ', num2str(qzero,2),];
        vec_qqsq = [vec_qqsq, ' & ', num2str(qzero^2/(2*n),2)];
        
        end

end        
        
