% this file simulates the marginal distribution of price changes
% first it computes them analytically
% then it verifies them by simulating them 
%( for the simulations the changes are all in the unit sphere, bar R = 1)
%  it produces figures and data for the tables in the paper
clear all; close all ;

nn=100; % maximum number of products for exact distributions

NO_sim = 1 ; %if set to 1, it does not perform simulations.

nn_sim = 10;  % maximum number of products for simulations

Std_Dp= 0.10;



% B=20;
% sigma = 0.10;
% psi_0 = 0.02;

TT = 50000 ; % number of draws for the marginal distribution

nbins = 100; % number of bins for the histograms

fign=0;

n=1; 

ybar =  Std_Dp^2* n  ; 

% ybar=sqrt(psi_0*nn*sigma^2*2*(nn+2)/B);
% HbarR=1.5; 
%p_vec =[-sqrt(HbarR):2*sqrt(HbarR)/199:sqrt(HbarR)];
p_vec =[-3*Std_Dp:6*Std_Dp/600:3*Std_Dp];

h = nan*zeros(max(size(p_vec)),1) ;

n_moments=4;
moments_level = nan*zeros(n_moments,nn);
moments_abs   = nan*zeros(n_moments,nn);
moments_abs_relm1 = nan*zeros(n_moments,nn);
moments_level_relm1 = nan*zeros(n_moments,nn);

tolquad=1.e-10;  %tolerance for quadrature.

ratio_meanabsDP_stdDP(1) =  1 ; 


for n=2:nn
    %n is number of products
    % psi=psi_0*n;
    
    ybar =  Std_Dp^2* n  ; 

    barR = ybar;
    
    barR_vec_CRTS(n) = barR; 
    
    % barR=1;
    
    const = gamma(n/2)/(gamma(1/2)*gamma((n-1)/2))/sqrt(barR);
    
    for ppi=1:length(p_vec);
        p = p_vec(ppi);
        if abs(p/sqrt(barR) ) < 1
            h(ppi,n) = const * ( 1 - (p/sqrt(barR))^2 )^((n-3)/2) ;
        else
            h(ppi,n) =0 ;
        end
    end
    
    for k=1:n_moments
        moments_level(k,n) = quad(@(p) fun_price_dist_multi_moment(p,barR,n,k),-sqrt(barR),sqrt(barR),tolquad) ;
        moments_abs(k,n)   = quad(@(p) 2*fun_price_dist_multi_moment(p,barR,n,k),0,sqrt(barR),tolquad) ;
    end
    
    % fraction 1points in the CDF
    p1 = 0.5  * moments_abs(1,n);
    p2 = 0.25 * moments_abs(1,n);
   
    
    % ratio of mean abs Dp  /   std(Dp)
    
    ratio_meanabsDP_stdDP(n) =  moments_abs(1,n)/ (moments_level(2,n))^.5 ; 
    
    % fraction of price changes smaller than 1/2 mean 
    fraction_leq_half_mean_abs(n) = quad(@(p) 2*fun_price_dist_multi_moment(p,barR,n,0),0,p1,tolquad) ;
    % fraction of price changes smaller than 1/2 mean 
    fraction_leq_quarter_mean_abs(n) = quad(@(p) 2*fun_price_dist_multi_moment(p,barR,n,0),0,p2,tolquad) ;
    
    
end
fontxh=16;

fign=fign+1;
figure(fign); 
ph=plot(p_vec,h(:,2),p_vec,h(:,3),p_vec,h(:,4));
maxa = max(max(max(h(:,2)),max(h(:,3))),max(h(:,4))) ;
nmax = 4;
% maxp = (sigma^2*psi_0*nmax*2*(nmax+2)/B)^(1/4) ;
maxp =2*Std_Dp; 
mina = 0;
axis([-1.05*maxp 1.05*maxp 0 20]);
%th=title({'Density h(\cdot) of the price changes for n=2,3,4 products', ...
%    ['keeping \psi_0 \sigma^2 / B constant']}) ;
xh=xlabel('values of the price change, in logs','Interpreter','Latex');
%set(xh,'Fontsize',fontxh);
legend('n = 2', 'n = 3','n = 4', 'Location','North') ;

% legend(['f = ' num2str(fvec(p1))],['f = ' num2str(fvec(p2))],...
%     ['f = ' num2str(fvec(p3))],['f = ' num2str(fvec(p4))]);

set(ph,'LineWidth',3);
set(gca,'FontSize',15);
%set(th,'FontSize',15);
set(xh,'FontSize',22);
%set(yh,'FontSize',12);

orient landscape 
%print -depsc .\fig1_distribution_n234


fign=fign+1;
figure(fign); 
ph=plot(p_vec,2*h(:,2),p_vec,2*h(:,3),p_vec,2*h(:,4));
maxa = 2*max(max(max(h(:,2)),max(h(:,3))),max(h(:,4))) ;
nmax = 4;
%maxp = (sigma^2*psi_0*nmax*2*(nmax+2)/B)^(1/4) ;
mina = 0;
axis([0 1.05*maxp 0 1.05*maxa]);
%th=title({'Density 2 h(\cdot) of the absolute value price changes for n=2,3,4 products', ...
%    ['keepping \psi_0 \sigma^2 / B constant']}) ;
xh=xlabel('absolute values of the price change, in logs','Interpreter','Latex'); 
legend('n = 2', 'n = 3','n = 4', 'Location','North') ;
set(ph,'LineWidth',3);
set(gca,'FontSize',15);
%set(th,'FontSize',15);
set(xh,'FontSize',18);
%set(yh,'FontSize',12);
orient landscape 
%print -depsc .\fig1_distribution_n234_abs


fign=fign+1;
nnn=min(50,nn);
n_sec=10;
std_nn=  sqrt(barR_vec_CRTS(nn)/nn);
figure(fign); 
ph=plot(p_vec,h(:,5),p_vec,h(:,n_sec),p_vec,h(:,nnn),p_vec,normpdf(p_vec,0,std_nn),'k --');
maxa = max(max(max(h(:,5)),max(h(:,n_sec))),max(h(:,nnn))) ;
nmax = 7;
maxp =3*Std_Dp; 
mina = 0;
axis([-1.05*maxp 1.05*maxp 0 1.05*maxa]);
% th=title({'Density h(\cdot) of the price changes for n=5,6,7 products', ...
%    'keepping \psi \sigma^2 / B constant'});
xh= xlabel('values of the price change, in logs','Interpreter','Latex');
legend('n = 5', ['n = ',num2str(n_sec,2)],['n = ',num2str(nnn,2)],'normal', 'Location','NorthEast') ;
set(ph,'LineWidth',3);
set(gca,'FontSize',15);
%set(th,'FontSize',15);
set(xh,'FontSize',22);
%set(yh,'FontSize',12);
orient landscape 
%print -depsc .\fig1_distribution_n567


fign=fign+1;
figure(fign); 
ph=plot(p_vec,2*h(:,5),p_vec,2*h(:,6),p_vec,2*h(:,7));
maxa = 2*max(max(max(h(:,5)),max(h(:,6))),max(h(:,7))) ;
nmax = 7;
maxp =3*Std_Dp; 
mina = 0;
axis([0  1.05*maxp 0 1.05*maxa]);
% th=title({'Density 2 h(\cdot) of the absolute price changes for n=5,6,7 products', ...
%    'keepping \psi \sigma^2 / B constant'}) ;
xh= xlabel('absolute value of the price change, in logs','Interpreter','Latex');
legend('n = 5', 'n = 6','n = 7', 'Location','NorthEast') ;
set(ph,'LineWidth',3);
set(gca,'FontSize',15);
%set(th,'FontSize',15);
set(xh,'FontSize',18);
%set(yh,'FontSize',12);
orient landscape 
%print -depsc .\fig1_distribution_n567_abs


for k=1:n_moments
    % moments ratio of k/2 power w.r.t. 2nd moment for each n
    moments_level_relm2(k,2:nn) = moments_level(k,2:nn)./ (moments_level(2,2:nn).^(k/2)) ;
    
    moments_abs_relm1(k,2:nn) = moments_abs(k,2:nn)./ (moments_abs(1,2:nn).^k) ;
    % moments relative to the 1 moments to the k for each n
end


std_abs = zeros(1,nn);
std_abs(2:nn) = (moments_abs(2,2:nn) - (moments_abs(1,2:nn).^2)).^(1/2);

std_level = zeros(1,nn);
std_level(2:nn) = (moments_level(2,2:nn) - (moments_level(1,2:nn).^2)).^(1/2);

std_mean_level = std_level ./ moments_level(1,:);

std_mean_abs   = std_abs ./ moments_abs(1,:);

kurtosis_level = nan*zeros(1,nn);

kurtosis_level(2:nn) = moments_level(4,2:nn) ./ (moments_level(2,2:nn).^2) ;

std_level_theory = zeros(1,nn);
std_level_theory = barR_vec_CRTS.^(0.5)./([1:1:nn].^(0.5)) ;

kurtosis_level_theory = 3*[1:1:nn]./([2+1:1:2+nn]) ;

exp_abs_theory_exact = nan*zeros(1,nn); 
exp_abs_theory_exact(2:nn) = (barR_vec_CRTS(2:nn).^(0.5))./([1:1:nn-1]/2 .* beta([1:1:nn-1]/2,1/2)) ;

exp_abs_theory_approx = (barR_vec_CRTS.^(0.5))./([1:1:nn].^(0.5)) *sqrt(2/pi) .* ((1+ 1.1*0.5./[1:1:nn]).^(0.5)) ;
    
std_mean_abs_approx = ( (pi/2)*(2*[1:1:nn]./(1.1+2*[1:1:nn])) -1).^(0.5) ;

std_mean_abs_theory = ( ( 0.5*[0:1:nn-1] .*beta([0:1:nn-1]/2,1/2) ).^2./[1:1:nn] - 1 ).^(0.5) ;

% 
% fign=fign+1;
% figure(fign); plot([1:1:n_moments],moments_abs_relm1(:,2:nn));
% title('moments of marginal distribution of absolute value of price changes for n=2,3,...,10')
% ylabel('moment of abs. value price change, relative to 1st moment')
% xlabel('number of products n')


% For table set nn=30 and use the following commands

high_nn=50;
sec_high_nn = 30;


if nn>= high_nn

 disp(' '); 
 disp([' mean(|D(p)| / std( D(p) ) for n= 2, 3, 4, 6, 10, 20, ',num2str(sec_high_nn,2),' , ',num2str(high_nn,2)]);
 disp(num2str(ratio_meanabsDP_stdDP([2 3 4 6 10 20 30 50]),2));
            
    
disp(' ');

disp(['std(|D(p)|/mean(|D(p)|) for n= 2, 3, 4, 6, 10, 20, ',num2str(sec_high_nn,2),' , ',num2str(high_nn,2)]);

disp(num2str([std_mean_abs(2:6), std_mean_abs(10),std_mean_abs(sec_high_nn),std_mean_abs(high_nn)],2));

disp(' ');

disp(['kurtosis of D(p) for n= 2, 3, 4, 6, 10, 20, ',num2str(sec_high_nn,2),' , ',num2str(high_nn,2)]);

disp(num2str([kurtosis_level(2:6), kurtosis_level(10),kurtosis_level(sec_high_nn),kurtosis_level(high_nn)],2));

disp(' ');

disp(['Fraction of |D(p)| < 1/2 mean|D(p)| for n= 2, 3, 4, 6, 10, 20, ',num2str(sec_high_nn,2),' , ',num2str(high_nn,2)]);

disp(num2str([fraction_leq_half_mean_abs(2:6), fraction_leq_half_mean_abs(10),...
    fraction_leq_half_mean_abs(sec_high_nn),fraction_leq_half_mean_abs(high_nn)],2));

disp(' ');

disp(['Fraction of |D(p)| < 1/4 mean|D(p)|  for n= 2, 3, 4, 6, 10, 20, ',num2str(sec_high_nn,2),' , ',num2str(high_nn,2)]);

disp(num2str([fraction_leq_quarter_mean_abs(2:6), fraction_leq_quarter_mean_abs(10),...
    fraction_leq_quarter_mean_abs(sec_high_nn),fraction_leq_quarter_mean_abs(high_nn)],2));

disp(' ') ;

end


if NO_sim ~= 1
    nn_old = nn;
    nn=nn_sim;
    
    
    for j=2:nn
        
        n=j; %number of products
        
        
        psi=psi_0*n;
        barR = sqrt(psi*sigma^2*2*(n+2)/B);
        
        T = round(TT/n) ;
        
        ZZ = randn(T,n);
        
        for t=1:T
            z(t,1:n) = ZZ(t,1:n)/norm(ZZ(t,1:n));
        end
        
        XX=zeros(T*n,1);
        %creates marginals
        for i=1:n
            XX(T*(i-1)+1:T*(i-1)+T,1) = ZZ(1:T,i);
            x(T*(i-1)+1:T*(i-1)+T,1)  = z(1:T,i) * sqrt(barR) ;
        end
        
        
        % figure(fign+1);  hist(Z,nbins); title(['marginal of Z, normal, n= ',num2str(n,2)]) ;
        
        fign=fign+1;
        figure(fign) ; hist(x,nbins); title(['marginal for normalized price changes, n= ',num2str(n,2)]) ;
        fign=fign+1;
        figure(fign) ; hist(abs(x),nbins); title(['marginal for absolute of normalized price changes, n= ',num2str(n,2)]) ;
        
        if n == 2
            fign=fign+1;
            figure(fign); plot(z(:,1),z(:,2),'o'); title(['z in circle, n= ',num2str(n,2)])
            
        end
        
        if n == 3
            fign=fign+1;
            figure(fign); plot3(z(:,1),z(:,2),z(:,3),'o'); title(['z in sphere, n= ',num2str(n,2)])
            
        end
        
        clear x XX z ZZ
        
    end
    
         nn=nn_old;
    
end % if
